
package jp.gr.java_conf.koto.notavacc.generator;

import java.util.*;
import java.io.*;
import java.math.*;
import jp.gr.java_conf.koto.io.*;
import jp.gr.java_conf.koto.notavacc.*;
import jp.gr.java_conf.koto.notavacc.util.*;
import jp.gr.java_conf.koto.notavacc.dfa.*;
import jp.gr.java_conf.koto.notavacc.dfa.DeterministicFiniteAutomaton.*;
import jp.gr.java_conf.koto.notavacc.lrg.*;
import jp.gr.java_conf.koto.notavacc.lrg.LRTable.*;
import jp.gr.java_conf.koto.notavacc.types.*;
import jp.gr.java_conf.koto.notavacc.parser.Original.*;
import jp.gr.java_conf.koto.notavacc.parser.Parser.*;
import jp.gr.java_conf.koto.notavacc.parser.Parser.Root;

public class JavaCodeGenerator extends CodeGenerator {
    private Root root;
    private DeterministicFiniteAutomaton dfa;
    private LRTable lrTable;
    private double targetVersion;
    private String manualLocale;
    private boolean javadocForManual;
    private boolean graphStructuredStack = false;
    public JavaCodeGenerator(Environment env, Root root, DeterministicFiniteAutomaton dfa, LRTable lrTable, TypeSystem typeSystem, double targetVersion) {
        super(env, root, dfa, lrTable, typeSystem);
        this.root = root;
        this.dfa = dfa;
        this.lrTable = lrTable;
        this.targetVersion = targetVersion;
        this.manualLocale = env.commandLine.argumentOf("javadoc-lang");
        if (this.manualLocale == null) {
            this.manualLocale = "en";
        }
        this.javadocForManual = env.commandLine.countOf("x-output-for-manual") > 0;
        
    }
    
    
    /*private static String toJavaConstantNameStyle(String name) {
        char prev = '_';
        StringBuffer result = new StringBuffer();
        for (int i = 0; i < name.length(); i++) {
            char ch = name.charAt(i);
            
            if (Character.isUpperCase(ch)) {
                if (Character.isLowerCase(prev)) {
                    result.append('_');             // e.g. FooURL to FOO_URL
                } else if (Character.isUpperCase(prev)) {
                    if (i + 1 < name.length()) {
                        char nextChar = name.charAt(i + 1);
                        if (Character.isLowerCase(nextChar)) {
                            result.append('_');     // e.g. URLBar to URL_BAR
                        }
                    }
                }
            }
            
            result.append(Character.toUpperCase(ch));
            prev = ch;
        }
        return result.toString();
    }*/
    
    private static String encodeForHTML(Object object) {
        return encodeForHTML(object.toString());
    }
    private static String encodeForHTML(String image) {
        StringBuffer result = new StringBuffer();
        for (int i = 0; i < image.length(); i++) {
            char ch = image.charAt(i);
            switch (ch) {
            default:
                if (ch <= ' ' || ch >= 0x7F) {
                    result.append("&#" + ((int) ch) + ";");
                } else {
                    result.append(ch);
                }
                break;
            case '<':
                result.append("&lt;");
                break;
            case '>':
                result.append("&gt;");
                break;
            case '&':
                result.append("&amp;");
                break;
            case '"':
                result.append("&quot;");
                break;
            }
        }
        return result.toString();
    }
    private static String toHexCode(char ch) {
        String hex = "000" + Integer.toString(ch, 16).toUpperCase(Locale.ENGLISH);
        hex = hex.substring(hex.length() - 4);
        return hex;
    }
    
    private static String quoted(char ch) {
        StringBuffer quoted = new StringBuffer("\'");
        quoted(quoted, ch);
        quoted.append("\'");
        return quoted.toString();
    }
    private static String quoted(String str) {
        return quoted(str, true);
    }
    private static String quoted(String str, boolean quote) {
        StringBuffer quoted = new StringBuffer();
        if (quote)
            quoted.append("\"");
        for (int i = 0; i < str.length(); i++) {
            char ch = str.charAt(i);
            quoted(quoted, ch);
        }
        if (quote)
            quoted.append("\"");
        return quoted.toString();
    }
    private static void quoted(StringBuffer result, char ch) {
        switch (ch) {
        default:
            if (0x0020 <= ch && ch <= 0x007E) {
                result.append(ch);
            } else {
                result.append("\\u").append(toHexCode(ch));
            }
            break;
        case '\n':   result.append("\\n");      break;
        case '\t':   result.append("\\t");      break;
        case '\b':   result.append("\\b");      break;
        case '\r':   result.append("\\r");      break;
        case '\f':   result.append("\\f");      break;
        case '\\':   result.append("\\\\");     break;
        case '\'':   result.append("\\\'");     break;
        case '\"':   result.append("\\\"");     break;
        }
    }
    
    
    private Map replacements = new HashMap();
    private void addReplacement(String word, String replacement) {
        if (replacements.containsKey(word))
            throw new IllegalArgumentException(word);
        replacements.put(word, replacement /*+ "_unique"*/);
    }
    private String replace(String str) {
        StringBuffer result = new StringBuffer();
        int index = 0;
        int mode = -1;  // -1 for initial state, 0 for identifiers, 1 for the other tokens.
        for (int i = 0;; i++) {
            char ch;
            if (i < str.length())
                ch = str.charAt(i);
            else
                ch = '\uFFFF';
            boolean identifierPart = Character.isJavaIdentifierPart(ch);
            int nextMode = identifierPart ? 0 : 1;
            if (mode != -1 && (ch == '\uFFFF' || nextMode != mode)) {
                String sub = str.substring(index, i);
                if (mode == 0) {
                    String newSequence = (String) replacements.get(sub);
                    if (newSequence != null)
                        sub = newSequence;
                }
                result.append(sub);
                index = i;
            }
            mode = nextMode;
            if (i >= str.length())
                break;
        }
        
        return result.toString();
    }
    private String replaceTypeParameter(String str) {
        StringBuffer result = null;
        int start = 0;
        for (;;) {
            int index = str.indexOf("$<", start);
            if (index < 0)
                break;
            if (result == null)
                result = new StringBuffer();
            result.append(str.substring(start, index));
            index++;
            int level = 0;
            while (index < str.length()) {
                char ch = str.charAt(index++);
                if (ch == '<') {
                    level++;
                } else if (ch == '>') {
                    level--;
                }
                if (targetVersion >= 1.5)
                    result.append(ch);
                if (level == 0)
                    break;
            }
            start = index;
        }
        if (result == null)
            return str;
        result.append(str.substring(start));
        return result.toString();
    }
    
    
    private List storePrinted = new LinkedList();
    private void println(String str) {
        storePrinted.add(str);
    }
    
    private void flush() throws IOException {
        Iterator it = storePrinted.iterator();
        while (it.hasNext()) {
            String str = (String) it.next();
            it.remove();
            
            str = replace(str);
            str = replaceTypeParameter(str);
            outputLine(str);
        }
    }
    
    private SafePrintWriter writer;
    private int indent = 0;
    private int temporalIndent = -1;
    private void outputLine(String str) throws IOException {
        str = str.trim();
        
        if (str.startsWith("}"))
            indent--;
        
        if (temporalIndent == -1)
            temporalIndent = indent;
        
        if (str.startsWith("case ") || str.startsWith("default:") || str.startsWith("default "))
            temporalIndent--;
        
        for (int i = 0; i < temporalIndent; i++)
            writer.print("    ");
        writer.println(str);
        temporalIndent = -1;
        
        if (str.endsWith("{")) {
            indent++;
        } else {
            if (str.startsWith("if ") || str.startsWith("while ") || str.startsWith("do ") || str.startsWith("for ")
                        || str.startsWith("if(") || str.startsWith("while(") || str.startsWith("do(") || str.startsWith("for(")
                        || str.equals("else") || str.startsWith("else "))
                temporalIndent = indent + 1;
        }
    }
    
    
    private String getBaseName(File file) {
        String name = file.getName();
        int index = name.lastIndexOf(".");
        if (index > 0)
            name = name.substring(0, index);
        return name;
    }
    
    protected void generateCode(File file, Tables tables, boolean dryRun) throws IOException {
        if (!dryRun) {
            if (environment.destinationEncoding == null) {
                writer = new SafePrintWriter(new FileOutputStream(file));
            } else {
                writer = new SafePrintWriter(new OutputStreamWriter(new FileOutputStream(file), environment.destinationEncoding));
            }
        } else {
            writer = new SafePrintWriter(new StringWriter());
        }
        try {
            String name = getBaseName(environment.processingFile);
            generateCode(file, name, tables);
            flush();
        } finally {
            writer.close();
        }
    }
    
    private Map typeToName = new HashMap();
    private String getTypeName(Type type) {
        String result = (String) typeToName.get(type);
        if (result == null) {
            if (type instanceof ArrayType) {
                if (targetVersion >= 1.5) {
                    ArrayType arrayType = (ArrayType) type;
                    result = "java.util.List<" + getTypeName(arrayType.getComponentType()) + ">";
                } else {
                    result = "java.util.List";
                }
            } else if (type instanceof NodeType) {
                result = "$Parser.$Node";
            } else if (type instanceof TokenType) {
                result = "$Parser.$Token";
            } else if (type instanceof ObjectType) {
                result = "java.lang.Object";
            } else {
                UserDefinedType userDefinedType = (UserDefinedType) type;
                result = "$Parser." + userDefinedType.getName();
            }
            typeToName.put(type, result);
        }
        return result;
    }
    
    private static final Object RESERVED_NAME = new Object();
    private void reserve(String name, MultiMap namespace) {
        namespace.add(name, RESERVED_NAME);
        addReplacement("$" + name, name);
    }
    private void allocate(String name, MultiMap namespace) {
        String alocatedName = "__" + name + "__notavacc_reserved";
        namespace.add(alocatedName, RESERVED_NAME);
        addReplacement("$" + name, alocatedName);
    }
    
    private void generateCode(File file, String topLevelClassName, Tables tables) throws IOException {
        logger.verbose(environment.processingFile, "''{0}'' EBNF productions.", new Integer(tables.typeNonterminals().size() + tables.aliasNonterminals().size()));
        
        MultiMap topTypeNamespace = new MultiMap();
        MultiMap topFieldNamespace = new MultiMap();
        MultiMap topMethodNamespace = new MultiMap();
        MultiMap nodeFieldNamespace = new MultiMap();
        MultiMap nodeNoArgumentMethodNamespace = new MultiMap();
        MultiMap nodeMethodNamespace = new MultiMap();
        
        final boolean visitorSupport = true;
        final boolean visitorWithReflection = true;
        final boolean replacementSupport = true;
        
        String packageName = root.getPackageName();
        addReplacement("$packageName", packageName);
        addReplacement("$Parser", topLevelClassName);
        topTypeNamespace.add(topLevelClassName, topLevelClassName);
        topTypeNamespace.add("java", RESERVED_NAME);
        topFieldNamespace.add("java", RESERVED_NAME);
        topMethodNamespace.add("java", RESERVED_NAME);
        nodeFieldNamespace.add("java", RESERVED_NAME);
        nodeNoArgumentMethodNamespace.add("java", RESERVED_NAME);
        nodeMethodNamespace.add("java", RESERVED_NAME);
        if (targetVersion >= 1.4) {
            addReplacement("$CharSequence", "java.lang.CharSequence");
            addReplacement("$AssertionError", "java.lang.AssertionError");
            addReplacement("$subSequence", "subSequence");
            addReplacement("$final_avoid_jdk12_bug", "final");
            addReplacement("$protected_avoid_jdk12_bug", "protected");
            addReplacement("$assert", "assert");
        } else {
            addReplacement("$CharSequence", "java.lang.StringBuffer");
            addReplacement("$AssertionError", "java.lang.RuntimeException");
            addReplacement("$subSequence", "substring");
            addReplacement("$final_avoid_jdk12_bug", " ");
            addReplacement("$protected_avoid_jdk12_bug", "public");
            addReplacement("$assert", "// ");
        }
        
        final int labelWordCount = tables.labelWordCount();
        
        if (javadocForManual) {
            addReplacement("$Type", "Type");
            addReplacement("$parseType", "parseType");
            addReplacement("$label", "label");
        }
        
]]<<
        #if (packageName != null)
            package $packageName;
        
        #if (manualLocale.equals("ja")) {
            /**
             * ￢＜＞＜∪∪ (notavaCC) >>[[ encodeForHTML(Version.VERSION) ]]<< によって出力された構文表現です。通常 >>[[ encodeForHTML(Version.LICENSE) ]]<< にしたがってライセンスされます。
            #if (javadocForManual) {
                 * <p>
                 * $Parser は、￢＜＞＜∪∪へ入力されたファイルの名前から、拡張子を除いたものです。
                 * パッケージは、<i><a href="../input.html#syntaxref:Root:PackageDeclaration" class="symbol">PackageDeclaration</a></i>によって指定されます。
                 * <tt>--target</tt> オプションの値が 1.4 より小さい場合、{@link java.lang.CharSequence} の代わりに {@link java.lang.StringBuffer} が、{@link java.lang.AssertionError} の代わりに {@link java.lang.RuntimeException} が使用されます。
            #}
             * <p>
             * メソッド
            #{
                #List parsableTypes = new ArrayList(tables.userDefinedTypes().size());
                #Iterator it1 = tables.userDefinedTypes().iterator();
                #while (it1.hasNext()) {
                    #UserDefinedType type = (UserDefinedType) it1.next();
                    #LRState initialState = (LRState) lrTable.symbolToInitialState().get(type.getTypeNonterminal());
                    #if (initialState != null) {
                        #parsableTypes.add(type);
                    #}
                #}
                #for (int i = 0; i < parsableTypes.size(); i++) {
                    #UserDefinedType type = (UserDefinedType) parsableTypes.get(i);
                    #if (i == parsableTypes.size() - 1) {
                         * {@link $Parser#parse>>[[ type.getName() ]]<<}
                    #} else if (i == parsableTypes.size() - 2) {
                         * {@link $Parser#parse>>[[ type.getName() ]]<<} および
                    #} else {
                         * {@link $Parser#parse>>[[ type.getName() ]]<<}、
                    #}
                #}
            #}
             * はテキストを構文解析し、抽象構文木を出力します。
             * ツリーのノードは、{@link $Node}のサブタイプのインスタンスになります。
             * <p>
             * このクラス、およびネストしたクラスは、<strong>同期化されません</strong>。
             * 
            #if (javadocForManual) {
                * @since    notavaCC 1.0
            #}
             */
        #} else {
            /**
             * A syntax representation, generated by &#xAC;&lt;&gt;&lt;&#x222A;&#x222A; (notavaCC) >>[[ encodeForHTML(Version.VERSION) ]]<<.  Licensed under >>[[ encodeForHTML(Version.LICENSE) ]]<< usually.
            #if (javadocForManual) {
                 * <p>
                 * $Parser is the name of the input file for the &#xAC;&lt;&gt;&lt;&#x222A;&#x222A; without the extension.
                 * The package of the class is given by <i><a href="../ja/input.html#syntaxref:Root:PackageDeclaration" class="symbol">PackageDeclaration</a></i>.
                 * If the value of the <tt>--target</tt> option is less than 1.4, {@link java.lang.CharSequence} and {@link java.lang.AssertionError} are replaced by {@link java.lang.StringBuffer} and {@link java.lang.RuntimeException}.
            #}
             * <p>
            #{
                #List parsableTypes = new ArrayList(tables.userDefinedTypes().size());
                #Iterator it1 = tables.userDefinedTypes().iterator();
                #while (it1.hasNext()) {
                    #UserDefinedType type = (UserDefinedType) it1.next();
                    #LRState initialState = (LRState) lrTable.symbolToInitialState().get(type.getTypeNonterminal());
                    #if (initialState != null) {
                        #parsableTypes.add(type);
                    #}
                #}
                #for (int i = 0; i < parsableTypes.size(); i++) {
                    #UserDefinedType type = (UserDefinedType) parsableTypes.get(i);
                    #if (i == parsableTypes.size() - 1) {
                         * {@link $Parser#parse>>[[ type.getName() ]]<<}
                    #} else if (i == parsableTypes.size() - 2) {
                         * {@link $Parser#parse>>[[ type.getName() ]]<<} and
                    #} else {
                         * {@link $Parser#parse>>[[ type.getName() ]]<<},
                    #}
                #}
                #if (parsableTypes.size() == 1) {
                     * parses a text and generates an abstruct syntax tree.
                #} else if (parsableTypes.size() > 1) {
                     * parse a text and generates an abstruct syntax tree.
                #}
            #}
             * A node of the tree is an instance of sub-types of {@link $Node}.
             * <p>
             * This class and any nested classes in this class are <strong>not synchronized</strong>.
             * 
            #if (javadocForManual) {
                * @since    notavaCC 1.0
            #}
             */
        #}
        public class $Parser {
            #if (manualLocale.equals("ja")) {
                /**
                 * パーザを構築します。
                #if (javadocForManual) {
                     * <i><a href="../input.html#syntaxref:Root:ConstructorScope" class="symbol">ConstructorScope</a></i> が与えられた場合、コンストラクタはデフォルトスコープになります。
                #}
                 * @since    notavaCC 1.0
                 */
            #} else {
                /**
                 * Constructs a parser.
                #if (javadocForManual) {
                     * <i><a href="../ja/input.html#syntaxref:Root:ConstructorScope" class="symbol">ConstructorScope</a></i> makes the constructor the default scope.
                #}
                 * @since    notavaCC 1.0
                 */
            #}
            #if (root.hasProtectedConstructor()) {
                $Parser() {
                }
            #} else {
                public $Parser() {
                }
            #}
            
            
            #Map symbolToJavaWord = new LinkedHashMap();
            #Map symbolToJavadocWord = new LinkedHashMap();
            #{
                #Terminal traditionalWhiteTerminal = new NamedTerminal("$WHITE_TOKEN");
                #Symbol[] symbols = tables.symbols();
                #for (int i = 0; i < symbols.length; i++) {
                    #Symbol symbol = symbols[i];
                    #int index = tables.getIndexOfSymbol(symbol);
                    #if (symbol instanceof EOFTerminal) {
                        #java.lang.String word = "EOF_TOKEN";
                        #symbolToJavaWord.put(symbol, "$Parser." + word);
                        #symbolToJavadocWord.put(symbol, "$Parser#" + word);
                        #if (manualLocale.equals("ja")) {
                            /**
                             * EOF をあらわします.
                             * @see     $Parser.$Token#getSymbolID()
                             * @since    notavaCC 1.0
                             */
                        #} else {
                            /**
                             * Describes the EOF.
                             * @see     $Parser.$Token#getSymbolID()
                             * @since    notavaCC 1.0
                             */
                        #}
                        public static final int >>[[ word ]]<< = >>[[ index ]]<<;
                        #topFieldNamespace.add(word, symbol);
                    #} else if (symbol.equals(traditionalWhiteTerminal)) {
                        #java.lang.String word = "WHITE_TOKEN";
                        #symbolToJavaWord.put(symbol, "$Parser." + word);
                        #symbolToJavadocWord.put(symbol, "$Parser#" + word);
                        /** Describes the white tokens, which are typically comments or white spaces. */
                        public static final int >>[[ word ]]<< = >>[[ index ]]<<;
                        #topFieldNamespace.add(word, symbol);
                    #} else if (symbol instanceof NamedTerminal) {
                        #NamedTerminal terminal = (NamedTerminal) symbol;
                        #java.lang.String word = "TOKEN_" + terminal.getName();
                        #symbolToJavaWord.put(symbol, "$Parser." + word);
                        #symbolToJavadocWord.put(symbol, "$Parser#" + word);
                        #if (manualLocale.equals("ja")) {
                            /**
                             * 終端記号 <code>>>[[ encodeForHTML(symbol) ]]<<</code> をあらわします。
                            #if (javadocForManual) {
                                 * このフィールドは、<i><a href="../input.html#syntax:TokenDefinition" class="symbol">TokenDefinition</a></i> で定義された終端記号ごとに１つずつ出力されます。<code>>>[[ encodeForHTML(symbol) ]]<<</code> は <i><a href="../input.html#syntaxref:TokenDefinition:IDENTIFIER" class="symbol">IDENITIFIER</a></i> で与えられる名前です。
                            #}
                             * @see     $Parser.$Token#getSymbolID()
                            #if (javadocForManual) {
                                * @since    notavaCC 1.0
                            #}
                             */
                        #} else {
                            /**
                             * Describes the terminal symbol <code>>>[[ encodeForHTML(symbol) ]]<<</code>.
                            #if (javadocForManual) {
                                 * A <i><a href="../ja/input.html#syntax:TokenDefinition" class="symbol">TokenDefinition</a></i> generates this field, and The word <code>>>[[ encodeForHTML(symbol) ]]<<</code> is given by <i><a href="../ja/input.html#syntaxref:TokenDefinition:IDENTIFIER" class="symbol">IDENITIFIER</a></i>.
                            #}
                             * @see     $Parser.$Token#getSymbolID()
                            #if (javadocForManual) {
                                * @since    notavaCC 1.0
                            #}
                             */
                        #}
                        public static final int >>[[ word ]]<< = >>[[ index ]]<<;
                        #topFieldNamespace.add(word, symbol);
                    #} else if (symbol instanceof TypeNonterminal) {
                        #TypeNonterminal nonterminal = (TypeNonterminal) symbol;
                        #symbolToJavaWord.put(symbol, nonterminal.getName() + ".$ID");
                        #symbolToJavadocWord.put(symbol, nonterminal.getName() + "#$ID");
                    #} else {
                        #assert (symbol instanceof StringTerminal) || (symbol instanceof AliasNonterminal) || (symbol instanceof AnonymousNonterminal);
                        #symbolToJavaWord.put(symbol, Integer.toString(tables.getIndexOfSymbol(symbol)));
                        #symbolToJavadocWord.put(symbol, Integer.toString(tables.getIndexOfSymbol(symbol)));
                    #}
                #}
            #}
            
            #Map tagToWord = new LinkedHashMap();
            #{
                #java.lang.String[] tags = tables.tags();
                #for (int i = 0; i < tags.length; i++) {
                    #java.lang.String tag = (java.lang.String) tags[i];
                    #java.lang.String word = "TAG_" + tag;
                    #tagToWord.put(tag, word);
                    public static final int >>[[ word ]]<< = >>[[ i ]]<<;
                    #topFieldNamespace.add(word, tag);
                #}
            #}
            
            
            /**
            #if (manualLocale.equals("ja")) {
                 * 抽象構文木のノードを構築します。
                 * <p>
                #if (javadocForManual) {
                     * このメソッドは、引数で与えられた情報からノードを構築するファクトリーメソッドです。このメソッドをオーバーライドすることで、ノードの構築をカスタマイズすることができます。
                #}
                 * <p>
                 * 構築すべきノードの型が、<code>symbolID</code>によって、次のように指定されます。
            #} else {
                 * Creates a node of an abstract syntax tree.
                 * <p>
                #if (javadocForManual) {
                     * This is a factory method to create a node from the information given by the arguments.  The user can override the method in order to customize the node creation.
                #}
                 * <p>
                 * The type of the node to create is described by <code>symbolID</code> as follows.
            #}
             * <table border=1>
             * <tr><th><code>symbolID</code></th><th>type to create</th></tr>
            #{
                #Iterator it0 = tables.userDefinedTypes().iterator();
                #while (it0.hasNext()) {
                    #UserDefinedType type = (UserDefinedType) it0.next();
                    #TypeNonterminal nonterminal = type.getTypeNonterminal();
                    #if (nonterminal != null) {
                         * <tr>
                         *   <td>{@link >>[[ symbolToJavadocWord.get(nonterminal) ]]<<}</td>
                         *   <td>{@link >>[[ getTypeName(type) ]]<<}</td>
                         * </tr>
                    #}
                #}
            #}
             * </table>
            #if (manualLocale.equals("ja")) {
                 * <p>
                 * このメソッドのデフォルトの実装は、<code>createNode(symbolID, parameters, false)</code> と等価です。
                 * 
                 * @param   symbolID
                 *      構築すべきノードの型。
                 * @param   parameters
                 *      ノード構築するのに必要な情報。
                 * @return
                 *      構築されたノード。
                 * @throws   $ParseException
                 *      構文解析対象のテキストが、満たすべき何らかの条件を満たしていない場合。
                #if (javadocForManual) {
                     * このメソッドでは、構文解析の対象であるテキストの誤りをチェックしてもかまいません。<code>$ParseException</code>は、誤りが発見された場合に<code>throw</code>ます。
                #}
            #} else {
                 * <p>
                 * The implementation by the <code>$Parser</code> is equivalent to <code>$createNode(symbolID, parameters, false)</code>.
                 * 
                 * @param   symbolID
                 *      the kind of the node to create.
                 * @param   parameters
                 *      the parameters to initialize the node.
                 * @return
                 *      a created node.
                 * @throws   $ParseException
                 *      if the text to parse doesn't satisfy some requirements.
            #}
             * @since    notavaCC 1.0
             */
            #reserve("createNode", topMethodNamespace);
            protected $Node $createNode(int symbolID, $NodeInitializationParameters parameters) throws $ParseException {
                return $createNode(symbolID, parameters, false);
            }
            
            #if (manualLocale.equals("ja")) {
                /**
                 * 抽象構文木のノードを構築します。
                 * 構築されたノードは、{@link $Parser.$Default} に含まれるクラスのインスタンスです。
                 * 
                 * @param   symbolID
                 *      構築すべきノードの型。{@link #$createNode(int, $Parser.$NodeInitializationParameters)} を参照。
                 * @param   parameters
                 *      ノード構築するのに必要な情報。
                 * @param   compact
                 *      ラベル付けされたノードだけを子に持つノードを構築するかどうか。このメソッドは <code>parameters</code>で指定された子を持つノードを構築しますが、<code>compact == true</code> の場合、その際にラベル付けされていないノードを除外します。
                 * @return
                 *      構築されたノード。
                 * @since    notavaCC 1.0
                 */
            #} else {
                /**
                 * Creates a node of an abstract syntax tree.  The created node is a instance of a class in {@link $Parser.$Default}.
                 * 
                 * @param   symbolID
                 *      the kind of the node to create.  See {@link #$createNode(int, $Parser.$NodeInitializationParameters)}.
                 * @param   parameters
                 *      the parameters to initialize the node.
                 * @param   compact
                 *      if <code>true</code>, the method creates a compact node, which have only labeled children.
                 * @return  a created node.
                 * @since    notavaCC 1.0
                 */
            #}
            protected final $Parser.$Default.$Node $createNode(int symbolID, $NodeInitializationParameters parameters, boolean compact) {
                switch(symbolID) {
                    #{
                        #Iterator it0 = tables.userDefinedTypes().iterator();
                        #while (it0.hasNext()) {
                            #UserDefinedType type = (UserDefinedType) it0.next();
                            #TypeNonterminal nonterminal = type.getTypeNonterminal();
                            #if (nonterminal != null) {
                                case >>[[ symbolToJavaWord.get(nonterminal) ]]<<:
                                    return new $Parser.$Default.>>[[ type.getName() ]]<<(parameters, compact);
                            #}
                        #}
                    #}
                }
                throw new $AssertionError("A node creation was failed.");
            }
            
            
            #if (manualLocale.equals("ja")) {
                /**
                 * ファイルに格納されている文字列を表す <code>$CharSequence</code> を得ます。
                 * <p>
                 * 
                 * @param   file
                 *      ファイル。
                 * @param   charsetName
                 *      文字セットの名前。デフォルトの文字セットを指定する場合 <code>null</code>。
                 * @return
                 *      ファイルに格納されている文字列を表す <code>$CharSequence</code>。
                 * @throws   java.io.IOException
                 *      ファイルへのアクセス中にエラーが発生した場合。
                 * @since    notavaCC 1.0
                 */
            #} else {
                /**
                 * Gets the <code>$CharSequence</code> that represents the text in the given file.
                 * <p>
                 * 
                 * @param   file
                 *      a file.
                 * @param   charsetName
                 *      the charset name of the file, or <code>null</code> for the default charset.
                 * @return
                 *      the <code>$CharSequence</code> that represents the text in the <code>file</code>.
                 * @throws   java.io.IOException
                 *      if some error is occured for the access to the <code>file</code>.
                 * @since    notavaCC 1.0
                 */
            #}
            #reserve("toCharSequence", topMethodNamespace);
            #if (targetVersion >= 1.4) {
                protected final $CharSequence $toCharSequence(java.io.File file, java.lang.String charsetName) throws java.io.IOException {
                    java.io.FileInputStream fis = null;
                    java.io.InputStreamReader reader = null;
                    java.nio.channels.FileChannel fc = null;
                    try {
                        fis = new java.io.FileInputStream(file);
                        if (charsetName == null)
                            reader = new java.io.InputStreamReader(fis);
                        else
                            reader = new java.io.InputStreamReader(fis, charsetName);
                        java.nio.charset.Charset charset;
                        try {
                            charset = java.nio.charset.Charset.forName(reader.getEncoding());
                        } catch(java.nio.charset.UnsupportedCharsetException x) {
                            charset = null;
                        }
                        if (charset != null) {
                            java.nio.charset.CharsetDecoder decoder = charset.newDecoder();
                            fc = fis.getChannel();
                            int size = (int) fc.size();
                            java.nio.MappedByteBuffer bb = fc.map(java.nio.channels.FileChannel.MapMode.READ_ONLY, 0, size);
                            return decoder.decode(bb);
                        } else {
                            long length = file.length();
                            if (length <= 0)
                                length = 1;
                            else if (length > Integer.MAX_VALUE)
                                length = Integer.MAX_VALUE;
                            java.lang.StringBuffer result = new java.lang.StringBuffer((int) length);
                            char[] buffer = new char[1024];
                            for(;;) {
                                int size = reader.read(buffer);
                                if (size < 0)
                                    break;
                                result.append(buffer, 0, size);
                            }
                            return result;
                        }
                    } finally {
                        if (fc != null)
                            fc.close();
                        if (reader != null)
                            reader.close();
                        else if (fis != null)
                            fis.close();
                    }
                }
            #} else {
                protected final $CharSequence $toCharSequence(java.io.File file, java.lang.String charsetName) throws java.io.IOException {
                    java.io.FileInputStream fis = null;
                    java.io.InputStreamReader reader = null;
                    try {
                        fis = new java.io.FileInputStream(file);
                        if (charsetName == null)
                            reader = new java.io.InputStreamReader(fis);
                        else
                            reader = new java.io.InputStreamReader(fis, charsetName);
                        long length = file.length();
                        if (length <= 0)
                            length = 1;
                        else if (length > Integer.MAX_VALUE)
                            length = Integer.MAX_VALUE;
                        java.lang.StringBuffer result = new java.lang.StringBuffer((int) length);
                        char[] buffer = new char[1024];
                        for(;;) {
                            int size = reader.read(buffer);
                            if (size < 0)
                                break;
                            result.append(buffer, 0, size);
                        }
                        return result;
                    } finally {
                        if (reader != null)
                            reader.close();
                        else if (fis != null)
                            fis.close();
                    }
                }
            #}
            #if (manualLocale.equals("ja")) {
                /**
                 * 文字ストリームから得られる文字列を表す <code>$CharSequence</code> を得ます。
                 * <p>
                 * このメソッドは、文字ストリームの終端に達するまで文字列を読み出し、その文字列をあらわす <code>$CharSequence</code> を返します。
                 * 
                 * @param   reader
                 *      文字ストリーム。このストリームは <code>close()</code> されません。
                 * @return
                 *      文字ストリームから得られる文字列を表す <code>$CharSequence</code>。
                 * @throws   java.io.IOException
                 *      文字ストリームへのアクセス中にエラーが発生した場合。
                 * @since    notavaCC 1.0
                 */
            #} else {
                /**
                 * Gets the <code>$CharSequence</code> that represents the text from the given character stream.
                 * <p>
                 * This method reads characters from the stream until the end of the stream has been reached, and returns the <code>$CharSequence</code> that represents the characters.
                 * 
                 * @param   reader
                 *      a character stream.  This method will not <code>close</code> it.
                 * @return
                 *      the <code>$CharSequence</code> that represents the text from the <code>reader</code>.
                 * @throws   java.io.IOException
                 *      if some error is occured for the access to the <code>reader</code>.
                 * @since    notavaCC 1.0
                 */
            #}
            protected final $CharSequence $toCharSequence(java.io.Reader reader) throws java.io.IOException {
                java.lang.StringBuffer result = new java.lang.StringBuffer();
                char[] buffer = new char[1024];
                for(;;) {
                    int size = reader.read(buffer);
                    if (size < 0)
                        break;
                    result.append(buffer, 0, size);
                }
                return result;
            }
            
            
            #if (manualLocale.equals("ja")) {
                /**
                 * 字句解析器を構築します。
                 * <p>
                #if (javadocForManual) {
                     * このメソッドは、字句解析器を構築するファクトリーメソッドです。このメソッドをオーバーライドすることで、字句解析をカスタマイズすることができます。
                #}
                 * 
                 * @param   sourceName
                 *      解析すべきテキストの名前。これは、{@link $Parser.$Token#getSourceName()} の戻り値を設定するためだけに使用されます。
                 * @param   text
                 *      解析すべきテキスト。
                 * @param   tabStop
                 *      テキスト中で、タブ文字に対して何文字ごとの位置に移動すべきか。これは、{@link $Parser.$Token#getColumn()} の戻り値を設定するためだけに使用されます。
                 * @return
                 *      与えられたテキストを対象に字句解析を行う字句解析器。
                 * @throws   $ParseException
                 *      構文解析対象のテキストが、満たすべき何らかの条件を満たしていない場合。
                #if (javadocForManual) {
                     * このメソッドでは、構文解析の対象であるテキストの誤りをチェックしてもかまいません。<code>$ParseException</code>は、誤りが発見された場合に<code>throw</code>ます。
                #}
                 * @since    notavaCC 1.0
                 */
            #} else {
                /**
                 * Creates a lexical analyzer.
                 * <p>
                #if (javadocForManual) {
                     * This is a factory method to create a lexical analyzer.  The user can override the method in order to customize the lexical analysis.
                #}
                 * @param   sourceName
                 *      the name of the source, used only to set the result of {@link $Parser.$Token#getSourceName()}.
                 * @param   text
                 *      the text to analyze.
                 * @param   tabStop
                 *      the number that the tab character advances the cursor to the next column that is a multiple of.  <code>tabStop</code> is used only to set the result of {@link $Parser.$Token#getColumn()}.
                 * @return
                 *      the lexical analyzer that analyze the <code>text</code>.
                 * @throws   $ParseException
                 *      if the text to parse doesn't satisfy some requirements.
                 * @since    notavaCC 1.0
                 */
            #}
            #reserve("createLexicalAnalyzer", topMethodNamespace);
            protected $Parser.$LexicalAnalyzer $createLexicalAnalyzer(java.lang.String sourceName, $CharSequence text, int tabStop) throws $ParseException {
                return new $Parser.$Default.$LexicalAnalyzer(sourceName, text, tabStop);
            }
            
            
            #if (manualLocale.equals("ja")) {
                /**
                 * ノードの構築に必要な情報を <code>private</code>なフィールドで保持するオブジェクト。このクラスのコンストラクタは<code>private</code>です。
                 * <p>
                 * 
                 * @since   notavaCC 1.0
                 */
            #} else {
                /**
                 * Contains <code>private</code> fields that is used in order to create a node.  The constructor is <code>private</code>.
                 * <p>
                 * 
                 * @since   notavaCC 1.0
                 */
            #}
            #reserve("NodeInitializationParameters", topTypeNamespace);
            $protected_avoid_jdk12_bug static class $NodeInitializationParameters {
                private $NodeInitializationParameters() {
                }
                private $Node[] childNodes;
                private java.util.List labelsIDList;
            }
            #if (visitorSupport) {
                
                
                #if (manualLocale.equals("ja")) {
                    /**
                     * ツリーをトラバースしながら何らかの処理を行うオブジェクト。
                     * <p>
                     * <code>visit</code> メソッドのディスパッチはリフレクションによって行われるため、このクラスはオーバーライドすべきメソッドを持ちません。
                     * 
                     * @see     $Parser.$Node#$accept($Parser.$Visitor)
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * A visitor.
                     * <p>
                     * This class has no method to override because the visit methods are dispatched by reflection.
                     * 
                     * @see     $Parser.$Node#$accept($Parser.$Visitor)
                     * @since   notavaCC 1.0
                     */
                #}
                #reserve("Visitor", topTypeNamespace);
                public static class $Visitor {
                    /*
                    public void visit($Parser.$Node node) {
                    }
                    public void visit($Parser.$Token node) {
                    }
                    #{
                        #Iterator it0 = tables.userDefinedTypes().iterator();
                        #while (it0.hasNext()) {
                            #UserDefinedType type = (UserDefinedType) it0.next();
                            public void visit(>>[[ getTypeName(type) ]]<< node) {
                            }
                        #}
                    #}
                    */
                    
                    private $final_avoid_jdk12_bug java.util.Map$<java.lang.Class, java.lang.reflect.Method> typeToMethod;
                    public $Visitor() {
                        typeToMethod = new java.util.HashMap$<java.lang.Class, java.lang.reflect.Method>();
                        java.lang.Class clazz = this.getClass();
                        java.lang.reflect.Method[] methods = clazz.getMethods();
                        for (int i = 0; i < methods.length; i++) {
                            java.lang.reflect.Method method = methods[i];
                            if (!method.getDeclaringClass().equals($Parser.$Visitor.class)) {
                                if ((method.getModifiers() & java.lang.reflect.Modifier.PUBLIC) != 0) {
                                    if (method.getName().equals("visit") && method.getExceptionTypes().length == 0) {
                                        java.lang.Class[] parameterTypes = method.getParameterTypes();
                                        if (parameterTypes.length == 1) {
                                            java.lang.Class parameterType = parameterTypes[0];
                                            if ($Parser.$Node.class.isAssignableFrom(parameterType)) {
                                                method.setAccessible(true);
                                                typeToMethod.put(parameterType, method);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        
                    }
                    
                    #if (manualLocale.equals("ja")) {
                        /**
                         * （部分）木をトラバースします。
                         * <p>
                         * <code>node</code> とその子のノードを、行きがけ順で再帰的にトラバースします。ノードは、<code>public</code> で、名前が <code>visit</code> で、そのノードと互換性のある型の引数１つを取り、<code>throws</code> 節を持たないメソッドへ、リフレクションを用いて渡されます。
                         * 
                         * <p>
                         * <b>注意：</b>将来のバージョンでは、リフレクションを用いないように変更される可能性があります。
                         * 
                         * @param   node
                         *      トラバースする（部分）木の根。
                         * @since   notavaCC 1.0
                         */
                    #} else {
                        /**
                         * Traverses a (sub-)tree.
                         * <p>
                         * The <code>node</code> and the child nodes of it are traversed recursively (preorder traversal).
                         * A node is passed to the method of <code>this</code> visitor, using reflection,
                         * that is public, whose name is <code>visit</code>,
                         * that has only one argument whose type is compatible with <code>this</code> node,
                         * and that has no throws clause.
                         * <p>
                         * <b>Note:</b> Maybe it will be dispatched without reflection in the future versions.
                         * 
                         * @param   node
                         *      the root of the (sub-)tree to traverse.
                         * @since   notavaCC 1.0
                         */
                    #}
                    public final void visitAll($Parser.$Node node) {
                        {
                            java.lang.Object[] args = null;
                            java.util.Iterator it = typeToMethod.entrySet().iterator();
                            while (it.hasNext()) {
                                java.util.Map.Entry entry = (java.util.Map.Entry) it.next();
                                java.lang.Class parameterType = (java.lang.Class) entry.getKey();
                                if (parameterType.isInstance(node)) {
                                    java.lang.reflect.Method method = (java.lang.reflect.Method) entry.getValue();
                                    if (args == null)
                                        args = new java.lang.Object[] { node };
                                    try {
                                        //System.out.println(node.getClass() + " " + method);
                                        method.invoke(this, args);
                                    } catch (java.lang.IllegalAccessException x) {
                                        throw new $AssertionError(method + " and the enclosing class(es) should be public or accessible from $Parser.$Visitor."
                                            + "  Note that anonymous classes are not public.");
                                    } catch (java.lang.reflect.InvocationTargetException x) {
                                        java.lang.Throwable targetException = x.getTargetException();
                                        if (targetException instanceof Error) {
                                            throw (java.lang.Error) targetException;
                                        } else {
                                            throw (java.lang.RuntimeException) targetException;
                                        }
                                    }
                                }
                            }
                        }
                        
                        {
                            java.util.Iterator it = node.$getChildNodes().iterator();
                            while (it.hasNext()) {
                                $Parser.$Node child = ($Parser.$Node) it.next();
                                visitAll(child);
                            }
                        }
                    }
                }
            #}
            
            
            #if (manualLocale.equals("ja")) {
                /**
                 * 構文解析の結果として得られる抽象構文木のノード。
                 * <p>
                 * 
                #if (javadocForManual) {
                     * @see     $Parser.$Type
                     * @see     $Parser.Token
                     * @see     #$parseType
                #}
                 * @since   notavaCC 1.0
                 */
            #} else {
                /**
                 * A node of the abstract syntax tree that is the result of parsing.
                 * <p>
                 * 
                #if (javadocForManual) {
                     * @see     $Parser.$Type
                     * @see     $Parser.Token
                     * @see     #$parseType
                #}
                 * @since   notavaCC 1.0
                 */
            #}
            #reserve("Node", topTypeNamespace);
            public static interface $Node {
                #if (manualLocale.equals("ja")) {
                    /**
                     * 子のノードのリストを返します。
                    #if (javadocForManual) {
                         * <p>
                         * ￢＜＞＜∪∪が出力するコードによって作られる $Node 型のオブジェクトでは、リストは変更可能で、リストの要素は構文解析対象のテキストに出現した順番に並びます。しかし、ユーザが構築したノードがこの条件を満たす必要はありません。
                    #}
                     * 
                     * @return  子のノードのリスト。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the list of the child nodes of <code>this</code>.
                    #if (javadocForManual) {
                         * <p>
                         * For a $Node created by the program generated by &#xAC;&lt;&gt;&lt;&#x222A;&#x222A;, the list should be modifiable and the nodes should be ordered by its position in the parsed text.  However, the nodes constructed by the users of &#xAC;&lt;&gt;&lt;&#x222A;&#x222A; are not required to satisfy it.
                    #}
                     * 
                     * @return  the list of the child nodes of <code>this</code>.
                     * @since   notavaCC 1.0
                     */
                #}
                #reserve("getChildNodes", nodeNoArgumentMethodNamespace);
                public java.util.List$<$Parser.$Node> $getChildNodes();
                
                #if (visitorSupport) {
                    /**
                     * Traverses the (sub-)tree whose root is <code>this</code> node.
                     * <p>
                     * <code>this</code> node is passed to the <code>visitor</code>,
                     * then the child nodes are traversed recursively (preorder traversal).
                     * A node is passed to the methods of the <code>visitor</code>, using reflection,
                     * that is public, whose name is <code>visit</code>,
                     * that have only one argument whose type is compatible with <code>this</code> node,
                     * and that have no throws clause.
                     * 
                     * @deprecated
                     *      This method will be replaced by {@link $Parser.$Visitor#visitAll}.
                     * 
                     * @param   visitor
                     *      a visitor.
                     * @since    notavacc 1.0
                     */
                    #reserve("accept", nodeMethodNamespace);
                    public void $accept($Visitor visitor);
                #}
                
                #if (manualLocale.equals("ja")) {
                    /**
                     * 親のノードを返します。
                     * <p>
                     * このメソッドは、単純に {@link $Parser.$Node#$setParentNode} で与えられた値を返します。構文木を編集する場合、ユーザは、このメソッドが親ノードを返すように適切に {@link $Parser.$Node#$setParentNode} を呼び出さなければなりません。
                     * 
                     * @return
                            このノードが構文木の根の場合 <code>null</code>。そうでない場合、親のノード。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the parent node of <code>this</code>, or null if <code>this</code> node is the root of the tree.
                     * <p>
                     * This method simply returns the value the method {@link $Parser.$Node#$setParentNode} set, so the above specification should be maintained by the users of <code>this</code> object.
                     * 
                     * @return
                            the parent node of <code>this</code>, or null if and only if <code>this</code> node is the root of the tree.
                     * @since   notavaCC 1.0
                     */
                #}
                #reserve("getParentNode", nodeNoArgumentMethodNamespace);
                public $Node $getParentNode();
                #if (manualLocale.equals("ja")) {
                    /**
                     * {@link $Parser.$Node#$getParentNode} の戻り値を設定します。
                     * <p>
                     * 
                     * @param   parentNode
                     *      次回以降の {@link $Parser.$Node#$getParentNode} の戻り値
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Sets the result of {@link $Parser.$Node#$getParentNode}.
                     * <p>
                     * 
                     * @param   parentNode
                     *      the value {@link $Parser.$Node#$getParentNode} returns.
                     * @since   notavaCC 1.0
                     */
                #}
                #reserve("setParentNode", nodeMethodNamespace);
                public void $setParentNode($Node parentNode);
            } // interface $Node
            
            #if (manualLocale.equals("ja")) {
                /**
                 * トークン（構文解析対象のテキストの断片）。
                 * <p>
                 * トークンは構文解析対象のテキスト（ソース）の断片です。通常、トークンは字句解析器（{@link $Parser.$LexicalAnalyzer}）によって作られます。
                 * このクラスには、そのトークンがどのソースのどの位置の断片であるかをあらわすメソッド（{@link #getSourceName}、{@link #getIndex}、{@link #getLine}、{@link #getColumn}）が用意されていますが、これらが返す値の厳密な意味は、字句解析器などのそのトークンを構築するプログラムによって規定されます。また、これらのメソッドはサポートされていなくてもよく、その場合 {@link java.lang.UnsupportedOperationException} 例外が発生します。
                 * 
                 * @since   notavaCC 1.0
                 */
            #} else {
                /**
                 * A token, which is a fragment of a parsed text.
                 * <p>
                 * A token is a fragment of a parsed text (a source).  Tokens are usually constructed by {@link $Parser.$LexicalAnalyzer}s.  This class introduces the methods that represent what is the source of the token and where the token is placed in the source ({@link #getSourceName}, {@link #getIndex}, {@link #getLine}, {@link #getColumn}), however, the strict specification of the returned value is defined by the generator of the token, e.g., <code>$LexicalAnalyzer</code>s.  Also, these methods may be unsuported and may throw {@link java.lang.UnsupportedOperationException}.
                 * 
                 * @since   notavaCC 1.0
                 */
            #}
            #reserve("Token", topTypeNamespace);
            public static interface $Token extends $Node {
                #if (manualLocale.equals("ja")) {
                    /**
                     * このトークンの種類を返します。
                     * <p>
                     * 戻り値は、次のいずれかです。
                     * 
                     * <table border=1>
                     * <tr><th>戻り値</th><th>トークンの種類</th></tr>
                    #{
                        #Symbol[] symbols = tables.symbols();
                        #for (int i = 0; i < tables.superiorIndexOfTerminals(); i++) {
                            # Symbol symbol = symbols[i];
                             * <tr>
                             *   <td>{@link >>[[ symbolToJavadocWord.get(symbol) ]]<<}</td>
                             *   <td><tt>>>[[ encodeForHTML(symbol) ]]<<</tt></td>
                             * </tr>
                        #}
                    #}
                     * </table>
                     * 
                     * @return  このトークンの種類。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the kind of <code>this</code> token.
                     * <p>
                     * The result value should be one of the followings:</p>
                     * 
                     * <table border=1>
                     * <tr><th>value</th><th>described token</th></tr>
                    #{
                        #Symbol[] symbols = tables.symbols();
                        #for (int i = 0; i < tables.superiorIndexOfTerminals(); i++) {
                            # Symbol symbol = symbols[i];
                             * <tr>
                             *   <td>{@link >>[[ symbolToJavadocWord.get(symbol) ]]<<}</td>
                             *   <td><tt>>>[[ encodeForHTML(symbol) ]]<<</tt></td>
                             * </tr>
                        #}
                    #}
                     * </table>
                     * 
                     * @return  the kind of <code>this</code> token.
                     * @since   notavaCC 1.0
                     */
                #}
                public int getSymbolID();
                
                #if (manualLocale.equals("ja")) {
                    /**
                     * このトークンがホワイト・トークンの場合、<code>true</code>を返します。
                     * <p>
                    #if (!javadocForManual) {
                        #if (tables.firstIndexOfWhiteTerminals() < tables.superiorIndexOfWhiteTerminals()) {
                             * This method returns <code>true</code> if and only if <code>getSymbolID()</code> is one of the following:
                             * <ul>
                            #{
                                #Symbol[] symbols = tables.symbols();
                                #for (int i = tables.firstIndexOfWhiteTerminals(); i < tables.superiorIndexOfWhiteTerminals(); i++) {
                                    # Symbol symbol = symbols[i];
                                     *   <td>{@link >>[[ symbolToJavadocWord.get(symbol) ]]<<}</td>
                                #}
                            #}
                             * </ul>
                        #} else {
                             * This method always returns <code>false</code> because the grammar represented by $Parser has no white token.
                        #}
                    #}
                     * 
                     * @return  このトークンがホワイト・トークンの場合 <code>true</code>。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returnds <code>true</code> if <code>this</code> is a white token.
                     * <p>
                    #if (!javadocForManual) {
                        #if (tables.firstIndexOfWhiteTerminals() < tables.superiorIndexOfWhiteTerminals()) {
                             * This method returns <code>true</code> if and only if <code>getSymbolID()</code> is one of the following:
                             * <ul>
                            #{
                                #Symbol[] symbols = tables.symbols();
                                #for (int i = tables.firstIndexOfWhiteTerminals(); i < tables.superiorIndexOfWhiteTerminals(); i++) {
                                    # Symbol symbol = symbols[i];
                                     *   <td>{@link >>[[ symbolToJavadocWord.get(symbol) ]]<<}</td>
                                #}
                            #}
                             * </ul>
                        #} else {
                             * This method always returns <code>false</code> because the grammar represented by $Parser has no white token.
                        #}
                    #}
                     * 
                     * @return  <code>true</code> if <code>this</code> is a while token.
                     * @since   notavaCC 1.0
                     */
                #}
                public boolean isWhite();
                
                #if (manualLocale.equals("ja")) {
                    /**
                     * このトークンに対応する文字列を返します。
                     * <p>
                     * 文字列がユニコードエスケープのようなメタ表現を含む場合、戻り値は、表現されている文字列（例えば A）であり、表現（例えば \u005Cu0041）ではありません。そのようなメタ表現が含まれていない場合、このメソッドの戻り値は {@link #getOriginalImage()} と同じになります。
                     * 
                     * @return  このトークンに対応する文字列。
                     * @see     $Parser.$Token#getOriginalImage()
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the string <code>this</code> token corresponds to.
                     * <p>
                     * If the string contains some meta notation like Unicode escapes, the returned value is the represented string (e.g., A) rather than the representation (e.g., \u005Cu0041).  If not, the returned value is the same as {@link #getOriginalImage()}.
                     * 
                     * @return  the string <code>this</code> token corresponds to.
                     * @see     $Parser.$Token#getOriginalImage()
                     * @since   notavaCC 1.0
                     */
                #}
                public java.lang.String getImage();
                
                #if (manualLocale.equals("ja")) {
                    /**
                     * このトークンに対応する文字列を返します。
                     * <p>
                     * 文字列がユニコードエスケープのようなメタ表現を含む場合、戻り値は、表現（例えば \u005Cu0041）であり、表現されている文字列（例えば A）ではありません。そのようなメタ表現が含まれていない場合、このメソッドの戻り値は {@link #getImage()} と同じになります。
                     * 
                     * @return  このトークンに対応する文字列。
                     * @see     $Parser.$Token#getImage()
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the string <code>this</code> token corresponds to.
                     * <p>
                     * If the string contains some meta notation like Unicode escapes, the returned value is the representation (e.g., \u005Cu0041) rather than the represented string (e.g., A).  If not, the returned value is the same as {@link #getImage()}.
                     * 
                     * @return  the string <code>this</code> token corresponds to.
                     * @see     $Parser.$Token#getImage()
                     * @since   notavaCC 1.0
                     */
                #}
                public java.lang.String getOriginalImage();
                
                #if (manualLocale.equals("ja")) {
                    /**
                     * このトークンに対応するソースの名前を返します。
                     * 
                     * @return  ソースの名前。
                     * @throws   java.lang.UnsupportedOperationException
                     *      サポートされない場合。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the source name <code>this</code> token coressponds to.
                     * 
                     * @return  the source name <code>this</code> token coressponds to.
                     * @throws   java.lang.UnsupportedOperationException
                     *      if the method is not suported.
                     * @since   notavaCC 1.0
                     */
                #}
                public java.lang.String getSourceName() throws java.lang.UnsupportedOperationException;
                
                #if (manualLocale.equals("ja")) {
                    /**
                     * ソース上の、このトークンが始まった位置のインデックス（0 based）を返します。
                     * 
                     * @return  このトークンが始まった位置のインデックス。
                     * @throws   java.lang.UnsupportedOperationException
                     *      サポートされない場合。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the zero-based index number in the source where <code>this</code> token begins.
                     * 
                     * @return  the index number in the source where <code>this</code> token begins.
                     * @throws   java.lang.UnsupportedOperationException
                     *      if the method is not suported.
                     * @since   notavaCC 1.0
                     */
                #}
                public int getIndex() throws java.lang.UnsupportedOperationException;
                #if (manualLocale.equals("ja")) {
                    /**
                     * ソース上の、このトークンが始まった位置の行番号（1 based）を返します。
                     * 
                     * @return  このトークンが始まった位置の行番号。
                     * @throws   java.lang.UnsupportedOperationException
                     *      サポートされない場合。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the one-based line number in the source where <code>this</code> token begins.
                     * 
                     * @return  the line number in the source where <code>this</code> token begins.
                     * @throws   java.lang.UnsupportedOperationException
                     *      if the method is not suported.
                     * @since   notavaCC 1.0
                     */
                #}
                public int getLine() throws java.lang.UnsupportedOperationException;
                #if (manualLocale.equals("ja")) {
                    /**
                     * ソース上の、このトークンが始まった位置のカラム番号（1 based）を返します。
                     * 
                     * @return  このトークンが始まった位置のカラム番号。
                     * @throws   java.lang.UnsupportedOperationException
                     *      サポートされない場合。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the one-based column number in the source where <code>this</code> token begins.
                     * 
                     * @return  the column number in the source where <code>this</code> token begins.
                     * @throws   java.lang.UnsupportedOperationException
                     *      if the method is not suported.
                     * @since   notavaCC 1.0
                     */
                #}
                public int getColumn() throws java.lang.UnsupportedOperationException;
                
                #if (manualLocale.equals("ja")) {
                    /**
                     * トークンの {@linkplain #getOriginalImage イメージ}と、サポートされている場合その位置を返します。
                     * 例えば、<code>'identifier' (line 2, column 3)</code> や <code>'identifier'</code> のようになります。厳密なフォーマットは規定されません。
                     * 
                     * @return  トークンのイメージと位置
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the {@linkplain #getOriginalImage image} and the position of <code>this</code> token if supported.
                     * For instance, <code>'identifier' (line 2, column 3)</code> or <code>'identifier'</code>.
                     * The format is not specified strictly.
                     * 
                     * @return  the {@linkplain #getOriginalImage image} and the parenthesized position of <code>this</code> token.
                     * @since   notavaCC 1.0
                     */
                #}
                public java.lang.String toString();
            } // interface $Token
            
            
            #int publicParsableTypeCount = 0;
            #{
                #reserve("ID", nodeFieldNamespace);
                #Iterator it1 = tables.userDefinedTypes().iterator();
                #while (it1.hasNext()) {
                    #UserDefinedType type = (UserDefinedType) it1.next();
                    #String scope = type.isProtected() ? "protected" : type.isPrivate() ? "private" : "public";
                    #String methodScope = type.isProtectedParsable() ? "protected" : "public";
                    #LRState initialState = (LRState) lrTable.symbolToInitialState().get(type.getTypeNonterminal());
                    #if (initialState != null) {
                        #if (methodScope.equals("public"))
                            #publicParsableTypeCount++;
                        #if (manualLocale.equals("ja")) {
                            /**
                             * ファイルに含まれる <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> を構文解析します。
                            #if (javadocForManual) {
                                 * <p>
                                 * ファイルに含まれるテキストを、非終端記号 <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> をゴール記号（開始記号）とする文法に基づき構文解析します。
                                 * このメソッドは、<code>$parsable</code> な非終端記号 <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> に対して出力されます。<code>>>[[ encodeForHTML(type.getName()) ]]<<</code> が <code>$parsable-protected</code> の場合、このメソッドは <code>protected</code> になります
                            #}
                             * <p>
                             * このメソッドの呼び出しは、<code>parse>>[[ encodeForHTML(type.getName()) ]]<<(file.getName(), file, null, 8)</code> の呼び出しと等価です。
                             * 
                             * @param   file
                             *      構文解析対象のテキストを含むファイル。
                             * @return
                             *      抽象構文木
                             * @throws   java.io.IOException
                             *      フェイルへのアクセス中にエラーが発生した場合。
                             * @throws   $ParseException
                             *      ファイルに含まれるテキストが、満たすべき条件を満たしていなかった場合。
                            #if (javadocForManual) {
                                * @since    notavaCC 1.0
                            #}
                             */
                        #} else {
                            /**
                             * Parses a <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> in the given file.
                            #if (javadocForManual) {
                                 * <p>
                                 * This method parses a text in the given file according to the grammar whose start symbol is <code>>>[[ encodeForHTML(type.getName()) ]]<<</code>.
                                 * A <code>$parsable</code> nonterminal symbol <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> generates this method.  If <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> is <code>$parsable-protected</code>, this method is <code>protected</code>.
                            #}
                             * <p>
                             * This is equivalent to <code>parse>>[[ encodeForHTML(type.getName()) ]]<<(file.getName(), file, null, 8)</code>.
                             * 
                             * @param   file
                             *      the file including the text to parse.
                             * @return
                             *      the abstract syntax tree.
                             * @throws   java.io.IOException
                             *      if some error is occured for the access to the file.
                             * @throws   $ParseException
                             *      if the text in the file doesn't satisfy some requirements.
                            #if (javadocForManual) {
                                * @since    notavaCC 1.0
                            #}
                             */
                        #}
                        #topMethodNamespace.add("parse" + type.getName(), type);
                        >>[[ methodScope ]]<< >>[[ getTypeName(type) ]]<< parse>>[[ type.getName() ]]<<(java.io.File file) throws java.io.IOException, $ParseException {
                            return parse>>[[ type.getName() ]]<<(file.getName(), file, null, 8);
                        }
                        #if (manualLocale.equals("ja")) {
                            /**
                             * ファイルに含まれる <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> を構文解析します。
                            #if (javadocForManual) {
                                 * <p>
                                 * ファイルに含まれるテキストを、非終端記号 <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> をゴール記号（開始記号）とする文法に基づき構文解析します。
                                 * このメソッドは、<code>$parsable</code> な非終端記号 <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> に対して出力されます。<code>>>[[ encodeForHTML(type.getName()) ]]<<</code> が <code>$parsable-protected</code> の場合、このメソッドは <code>protected</code> になります
                            #}
                             * 
                             * @param   sourceName
                             *      解析すべきテキストの名前。これは、{@link $Parser.$Token#getSourceName()} の戻り値を設定するためだけに使用されます。
                             * @param   file
                             *      構文解析対象のテキストを含むファイル。
                             * @param   charsetName
                             *      文字セットの名前。デフォルトの文字セットを指定する場合 <code>null</code>。
                             * @param   tabStop
                             *      テキスト中で、タブ文字に対して何文字ごとの位置に移動すべきか。これは、{@link $Parser.$Token#getColumn()} の戻り値を設定するためだけに使用されます。
                             * @return
                             *      抽象構文木
                             * @throws   java.io.IOException
                             *      フェイルへのアクセス中にエラーが発生した場合。
                             * @throws   $ParseException
                             *      ファイルに含まれるテキストが、満たすべき条件を満たしていなかった場合。
                            #if (javadocForManual) {
                                * @since    notavaCC 1.0
                            #}
                             */
                        #} else {
                            /**
                             * Parses a <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> in the given file.
                            #if (javadocForManual) {
                                 * <p>
                                 * This method parses a text in the given file according to the grammar whose start symbol is <code>>>[[ encodeForHTML(type.getName()) ]]<<</code>.
                                 * A <code>$parsable</code> nonterminal symbol <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> generates this method.  If <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> is <code>$parsable-protected</code>, this method is <code>protected</code>.
                            #}
                             * 
                             * @param   sourceName
                             *      the name of the source, used only to set the result of {@link $Parser.$Token#getSourceName()}.
                             * @param   file
                             *      the file including the text to parse.
                             * @param   charsetName
                             *      the charset name of the file, or <code>null</code> for the default charset.
                             * @param   tabStop
                             *      the number that the tab character advances the cursor to the next column that is a multiple of.  <code>tabStop</code> is used only to set the result of {@link $Parser.$Token#getColumn()}.
                             * @return
                             *      the abstract syntax tree.
                             * @throws   java.io.IOException
                             *      if some error is occured for the access to the file.
                             * @throws   $ParseException
                             *      if the text in the file doesn't satisfy some requirements.
                            #if (javadocForManual) {
                                * @since    notavaCC 1.0
                            #}
                             */
                        #}
                        >>[[ methodScope ]]<< >>[[ getTypeName(type) ]]<< parse>>[[ type.getName() ]]<<(java.lang.String sourceName, java.io.File file, java.lang.String charsetName, int tabStop) throws java.io.IOException, $ParseException {
                            return parse>>[[ type.getName() ]]<<(sourceName, $toCharSequence(file, charsetName), tabStop);
                        }
                        #if (manualLocale.equals("ja")) {
                            /**
                             * 文字ストリームに含まれる <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> を構文解析します。
                            #if (javadocForManual) {
                                 * <p>
                                 * 文字ストリームの終端に達するまで文字列を読み出し、その文字列を 非終端記号 <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> をゴール記号（開始記号）とする文法に基づき構文解析します。
                                 * このメソッドは、<code>$parsable</code> な非終端記号 <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> に対して出力されます。<code>>>[[ encodeForHTML(type.getName()) ]]<<</code> が <code>$parsable-protected</code> の場合、このメソッドは <code>protected</code> になります
                            #}
                             * 
                             * @param   sourceName
                             *      解析すべきテキストの名前。これは、{@link $Parser.$Token#getSourceName()} の戻り値を設定するためだけに使用されます。
                             * @param   reader
                             *      解析すべきテキストを含む文字ストリーム。このストリームは <code>close()</code> されません。
                             * @param   tabStop
                             *      テキスト中で、タブ文字に対して何文字ごとの位置に移動すべきか。これは、{@link $Parser.$Token#getColumn()} の戻り値を設定するためだけに使用されます。
                             * @return
                             *      抽象構文木
                             * @throws   java.io.IOException
                             *      文字ストリームへのアクセス中にエラーが発生した場合。
                             * @throws   $ParseException
                             *      文字ストリームに含まれるテキストが、満たすべき条件を満たしていなかった場合。
                            #if (javadocForManual) {
                                * @since    notavaCC 1.0
                            #}
                             */
                        #} else {
                            /**
                             * Parses a <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> in the given character stream.
                            #if (javadocForManual) {
                                 * <p>
                                 * This method reads characters from the stream until the end of the stream has been reached,
                                 * and parses the text according to the grammar whose start symbol is <code>>>[[ encodeForHTML(type.getName()) ]]<<</code>.
                                 * A <code>$parsable</code> nonterminal symbol <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> generates this method.  If <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> is <code>$parsable-protected</code>, this method is <code>protected</code>.
                            #}
                             * 
                             * @param   sourceName
                             *      the name of the source, used only to set the result of {@link $Parser.$Token#getSourceName()}.
                             * @param   reader
                             *      a character stream.  This method will not <code>close</code> it.
                             * @param   tabStop
                             *      the number that the tab character advances the cursor to the next column that is a multiple of.  <code>tabStop</code> is used only to set the result of {@link $Parser.$Token#getColumn()}.
                             * @return
                             *      the abstract syntax tree.
                             * @throws   java.io.IOException
                             *      if some error is occured for the access to the file.
                             * @throws   $ParseException
                             *      if the text in the file doesn't satisfy some requirements.
                            #if (javadocForManual) {
                                * @since    notavaCC 1.0
                            #}
                             */
                        #}
                        >>[[ methodScope ]]<< >>[[ getTypeName(type) ]]<< parse>>[[ type.getName() ]]<<(java.lang.String sourceName, java.io.Reader reader, int tabStop) throws java.io.IOException, $ParseException {
                            return parse>>[[ type.getName() ]]<<(sourceName, $toCharSequence(reader), tabStop);
                        }
                        #if (manualLocale.equals("ja")) {
                            /**
                             * <code>$CharSequence</code> に含まれる <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> を構文解析します。
                            #if (javadocForManual) {
                                 * <p>
                                 * <code>$CharSequence</code> を 非終端記号 <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> をゴール記号（開始記号）とする文法に基づき構文解析します。
                                 * このメソッドは、<code>$parsable</code> な非終端記号 <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> に対して出力されます。<code>>>[[ encodeForHTML(type.getName()) ]]<<</code> が <code>$parsable-protected</code> の場合、このメソッドは <code>protected</code> になります
                            #}
                             * 
                             * @param   sourceName
                             *      解析すべきテキストの名前。これは、{@link $Parser.$Token#getSourceName()} の戻り値を設定するためだけに使用されます。
                             * @param   text
                             *      解析すべきテキストを含む <code>$CharSequence</code>。
                             * @param   tabStop
                             *      テキスト中で、タブ文字に対して何文字ごとの位置に移動すべきか。これは、{@link $Parser.$Token#getColumn()} の戻り値を設定するためだけに使用されます。
                             * @return
                             *      抽象構文木
                             * @throws   $ParseException
                             *      <code>text</code> に含まれるテキストが、満たすべき条件を満たしていなかった場合。
                            #if (javadocForManual) {
                                * @since    notavaCC 1.0
                            #}
                             */
                        #} else {
                            /**
                             * Parses a <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> in the given text.
                            #if (javadocForManual) {
                                 * <p>
                                 * This method parses the <code>text</code> according to the grammar whose start symbol is <code>>>[[ encodeForHTML(type.getName()) ]]<<</code>.
                                 * A <code>$parsable</code> nonterminal symbol <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> generates this method.  If <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> is <code>$parsable-protected</code>, this method is <code>protected</code>.
                            #}
                             * 
                             * @param   sourceName
                             *      the name of the source, used only to set the result of {@link $Parser.$Token#getSourceName()}.
                             * @param   text
                             *      the text to parse.
                             * @param   tabStop
                             *      the number that the tab character advances the cursor to the next column that is a multiple of.  <code>tabStop</code> is used only to set the result of {@link $Parser.$Token#getColumn()}.
                             * @return
                             *      the abstract syntax tree.
                             * @throws   $ParseException
                             *      if the text doesn't satisfy some requirements.
                            #if (javadocForManual) {
                                * @since    notavaCC 1.0
                            #}
                             */
                        #}
                        >>[[ methodScope ]]<< >>[[ getTypeName(type) ]]<< parse>>[[ type.getName() ]]<<(java.lang.String sourceName, $CharSequence text, int tabStop) throws $ParseException {
                            $Parser.$LexicalAnalyzer analyzer = $createLexicalAnalyzer(sourceName, text, tabStop);
                            return parse>>[[ type.getName() ]]<<(analyzer);
                        }
                        #if (manualLocale.equals("ja")) {
                            /**
                             * 字句解析器が出力するトークンの列を構文解析します。
                            #if (javadocForManual) {
                                 * <p>
                                 * 字句解析器が出力するトークンの列を 非終端記号 <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> をゴール記号（開始記号）とする文法に基づき構文解析します。
                                 * このメソッドは、<code>$parsable</code> な非終端記号 <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> に対して出力されます。<code>>>[[ encodeForHTML(type.getName()) ]]<<</code> が <code>$parsable-protected</code> の場合、このメソッドは <code>protected</code> になります
                            #}
                             * 
                             * @param   analyzer
                             *      トークンの列を与える字句解析器
                             * @return
                             *      抽象構文木
                             * @throws   $ParseException
                             *      <code>text</code> に含まれるテキストが、満たすべき条件を満たしていなかった場合。
                            #if (javadocForManual) {
                                * @since    notavaCC 1.0
                            #}
                             */
                        #} else {
                            /**
                             * Parses the sequence of tokens.
                            #if (javadocForManual) {
                                 * <p>
                                 * This method parses the sequence of the tokens given by the lexical analyzer according to the grammar whose start symbol is <code>>>[[ encodeForHTML(type.getName()) ]]<<</code>.
                                 * A <code>$parsable</code> nonterminal symbol <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> generates this method.  If <code>>>[[ encodeForHTML(type.getName()) ]]<<</code> is <code>$parsable-protected</code>, this method is <code>protected</code>.
                            #}
                             * 
                             * @param   analyzer
                             *      the lexical analyzer that gives the sequence of tokens to parse.
                             * @return
                             *      the abstract syntax tree.
                             * @throws   $ParseException
                             *      if the sequence of tokens doesn't satisfy some requirements.
                            #if (javadocForManual) {
                                * @since    notavaCC 1.0
                            #}
                             */
                        #}
                        >>[[ methodScope ]]<< >>[[ getTypeName(type) ]]<< parse>>[[ type.getName() ]]<<($Parser.$LexicalAnalyzer analyzer) throws $ParseException {
                            int initialState = >>[[ tables.getIndexOfState(initialState) ]]<<;
                            int classSymbolMaximum = >>[[ tables.superiorIndexOfClassNonterminals() - 1 ]]<<;
                            return (>>[[ getTypeName(type) ]]<<) $parse($Parser.$lrTable, $Parser.$reductionTable, $Parser.$tagTable, initialState, classSymbolMaximum, analyzer);
                        }
                    #}
                    #StringBuffer extendedTypes = new StringBuffer();
                    #{
                        #boolean first = true;
                        #Iterator inner = type.getDirectSuperTypes().iterator();
                        #while (inner.hasNext()) {
                            #Type superType = (Type) inner.next();
                            #if (first)
                                #first = false;
                            #else
                                #extendedTypes.append(", ");
                            #extendedTypes.append(getTypeName(superType));
                        #}
                    #}
                    #if (manualLocale.equals("ja")) {
                        /**
                         * 抽象構文木のノード <code>>>[[ encodeForHTML(type.getName()) ]]<<</code>.
                        #if (javadocForManual) {
                             * <p>
                             * このクラスは、１つの <i><a href="../input.html#syntax:TypeDefinition" class="symbol">TypeDefinition</a></i> 毎に１つ出力されます。>>[[ encodeForHTML(type.getName()) ]]<< は、<i><a href="../input.html#syntaxref:TypeDefinition:IDENTIFIER" class="symbol">IDENITIFIER</a></i> で与えられます。<i><a href="../input.html#syntax:TypeDefinition" class="symbol">TypeDefinition</a></i> が <code>$protected</code> か <code>$private</code> の場合、このクラスは <code>protected</code> か <code>private</code> になります。そうでない場合 <code>public</code> になります。
                        #}
                         * 
                            #if (javadocForManual) {
                                * @since    notavaCC 1.0
                            #}
                         */
                    #} else {
                        /**
                         * An abstract-syntax-tree node <code>>>[[ encodeForHTML(type.getName()) ]]<<</code>.
                         * 
                        #if (javadocForManual) {
                             * <p>
                             * A <i><a href="../ja/input.html#syntax:TypeDefinition" class="symbol">TypeDefinition</a></i> generates this class.  The word >>[[ encodeForHTML(type.getName()) ]]<< is given by <i><a href="../ja/input.html#syntaxref:TypeDefinition:IDENTIFIER" class="symbol">IDENITIFIER</a></i>.  If <i><a href="../ja/input.html#syntax:TypeDefinition" class="symbol">TypeDefinition</a></i> is <code>$protected</code> or <code>$private</code>, this class is <code>protected</code> or <code>private</code>.  Otherwise, it is <code>public</code>.
                        #}
                         * 
                            #if (javadocForManual) {
                                * @since    notavaCC 1.0
                            #}
                         */
                    #}
                    #topTypeNamespace.add(type.getName(), type);
                    >>[[ scope ]]<< static interface >>[[ type.getName() ]]<< extends >>[[ extendedTypes ]]<< {
                        #TypeNonterminal nonterminal = type.getTypeNonterminal();
                        #if (nonterminal != null) {
                            #if (manualLocale.equals("ja")) {
                                /**
                                 * この型の ID 値。
                                 * @see     $Parser#$createNode
                                 * @since   notavaCC 1.0
                                 */
                            #} else {
                                /**
                                 * The ID value of the type.
                                 * @see     $Parser#$createNode
                                 * @since   notavaCC 1.0
                                 */
                            #}
                            public static final int $ID = >>[[ tables.getIndexOfSymbol(nonterminal) ]]<<;
                        #}
                        #Iterator it2 = type.getDeclaredFields().iterator();
                        #while (it2.hasNext()) {
                            #Field field = (Field) it2.next();
                            #java.lang.String typeName = getTypeName(field.getType());
                            #java.lang.String name = field.getName();
                            #if (manualLocale.equals("ja")) {
                                #if (javadocForManual) {
                                    /**
                                     * <code>>>[[ encodeForHTML(name) ]]<<</code> でラベル付けされる子を返します。
                                     * <p>
                                     * ラベル付けされる子が高々１つしかない場合、このメソッドの戻り値の型は、ラベル付けされる子に共通の型で最も限定的なものになります。このとき、ラベル付けされる子が無ければ、戻り値は <code>null</code> になります。
                                     * <p>
                                     * ラベル付けされる子が複数になる可能性がある場合、このメソッドの戻り値の型は、ラベル付けされる子に共通の型で最も限定的なものの {@link java.util.List} になります。このとき、ラベル付けされる子が無ければ、戻り値は <code>isEmpty</code> になります。
                                     * <p>
                                     * <b>注意：</b>将来のバージョンでは、{@link java.util.List}ではなく配列を返すように変更される可能性があります。
                                     * 
                                     * @return
                                     *      <code>>>[[ encodeForHTML(name) ]]<<</code> でラベル付けされる子。
                                     * @since   notavaCC 1.0
                                     */
                                #} else {
                                    #if (field.getType() instanceof ArrayType) {
                                        #ArrayType at = (ArrayType) field.getType();
                                        /**
                                         * @return
                                         *      {@link >>[[ encodeForHTML(getTypeName(at.getComponentType())) ]]<<} のリスト。
                                         */
                                    #} else {
                                        /**
                                         */
                                    #}
                                #}
                            #} else {
                                #if (javadocForManual) {
                                    /**
                                     * Returns the child nodes labeled by <code>>>[[ encodeForHTML(name) ]]<<</code> in the notavaCC source.
                                     * <p>
                                     * If the number of the labeled children should be zero or one, the type of the result is the most specific common type.  In this case, if there is no labeled children, the result is <code>null</code>.
                                     * <p>
                                     * If the number of the labeled children may be greater than one, the type of the result is the {@link java.util.List} of the most specific common type.  In this case, if there is no labeled children, the result <code>isEmpty</code>.
                                     * <p>
                                     * <b>Note:</b> Maybe the result will be an array rather than {@link java.util.List} in the future versions.
                                     * 
                                     * @return
                                     *      the child nodes labeled by <code>>>[[ encodeForHTML(name) ]]<<</code>.
                                     * @since   notavaCC 1.0
                                     */
                                #} else {
                                    #if (field.getType() instanceof ArrayType) {
                                        #ArrayType at = (ArrayType) field.getType();
                                        /**
                                         * @return
                                         *      the list of {@link >>[[ encodeForHTML(getTypeName(at.getComponentType())) ]]<<}
                                         */
                                    #} else {
                                        /**
                                         */
                                    #}
                                #}
                            #}
                            #nodeNoArgumentMethodNamespace.add(name, name);
                            public >>[[ typeName ]]<< >>[[ name ]]<<();
                        #}
                    }
                #}
            #}
            
            
            #if (manualLocale.equals("ja")) {
                /**
                 * 解析対象のテキストが、何らかの誤りを持っていることをあらわします。
                #if (javadocForManual) {
                     * <p>
                     * ￢＜＞＜∪∪が出力されたコードがこの例外を発生させるのは、テキストが構文上の誤りを持っていた場合のみです。しかし、ユーザは、テキストが意味上の誤りを持っていた場合にこの例外を発生させてもかまいません。
                #}
                 * 
                 * @since   notavaCC 1.0
                 */
            #} else {
                /**
                 * Describes a error of a text.
                #if (javadocForManual) {
                     * <p>
                     * The program generated by &#xAC;&lt;&gt;&lt;&#x222A;&#x222A; throws this exception only if the parsed text has some syntactical error.  But, the users of &#xAC;&lt;&gt;&lt;&#x222A;&#x222A; may throw this exception if it has some semantic error.
                #}
                 * 
                 * @since   notavaCC 1.0
                 */
            #}
            #reserve("ParseException", topTypeNamespace);
            public static class $ParseException extends java.lang.Exception {
                private final java.lang.String sourceName;
                private final int index;
                private final int line;
                private final int column;
                
                #if (manualLocale.equals("ja")) {
                    /**
                     * 例外を構築します。
                     * 
                     * @param   msg
                     *            エラーの内容を表すメッセージ。
                     * @param   sourceName
                     *            エラーが発生したソースの名前。サポートされない場合 <code>null</code>。
                     * @param   index
                     *            エラーが発生した位置のインデックス。サポートされない場合 <code>-1</code>。
                     * @param   line
                     *            エラーが発生した位置の行番号。サポートされない場合 <code>-1</code>。
                     * @param   column
                     *            エラーが発生した位置のカラム番号。サポートされない場合 <code>-1</code>。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Constructs an exception.
                     * 
                     * @param   msg
                     *            a message to describes the error.
                     * @param   sourceName
                     *            the source name where the exception is occured, or null if not supported.
                     * @param   index
                     *            the index number where the exception is occured, or -1 if not supported.
                     * @param   line
                     *            the line number where the exception is occured, or -1 if not supported.
                     * @param   column
                     *            the column number where the exception is occured, or -1 if not supported.
                     * @since   notavaCC 1.0
                     */
                #}
                public $ParseException(java.lang.String msg, java.lang.String sourceName, int index, int line, int column) {
                    super(msg);
                    this.sourceName = sourceName;
                    this.index = index;
                    this.line = line;
                    this.column = column;
                }
                #if (manualLocale.equals("ja")) {
                    /**
                     * 例外を構築します。
                     * 
                     * @param   msg
                     *            エラーの内容を表すメッセージ。
                     * @param   position
                     *            エラーが発生した位置を表すトークン。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Constructs an exception.
                     * 
                     * @param   msg
                     *            a message to describes the error.
                     * @param   position
                     *            a token that describes the position of the error.
                     * @since   notavaCC 1.0
                     */
                #}
                public $ParseException(java.lang.String msg, $Parser.$Token position) {
                    super(msg);
                    
                    java.lang.String sourceName = null;
                    int index = -1;
                    int line = -1;
                    int column = -1;
                    
                    try {
                        sourceName = position.getSourceName();
                    } catch (java.lang.UnsupportedOperationException x) {
                    }
                    try {
                        index = position.getIndex();
                    } catch (java.lang.UnsupportedOperationException x) {
                    }
                    try {
                        line = position.getLine();
                    } catch (java.lang.UnsupportedOperationException x) {
                    }
                    try {
                        column = position.getColumn();
                    } catch (java.lang.UnsupportedOperationException x) {
                    }
                    
                    this.sourceName = sourceName;
                    this.index = index;
                    this.line = line;
                    this.column = column;
                }
                
                #if (manualLocale.equals("ja")) {
                    /**
                     * エラーが発生したソースの名前を返します。
                     * 
                     * @return  エラーが発生したソースの名前。
                     * @throws   java.lang.UnsupportedOperationException
                     *      サポートされない場合。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the source name where the exception is occured.
                     * 
                     * @return  the source name where the exception is occured.
                     * @throws   java.lang.UnsupportedOperationException
                     *      if the method is not suported.
                     * @since   notavaCC 1.0
                     */
                #}
                public final java.lang.String getSourceName() throws java.lang.UnsupportedOperationException {
                    if (sourceName == null)
                        throw new java.lang.UnsupportedOperationException();
                    return sourceName;
                }
                
                #if (manualLocale.equals("ja")) {
                    /**
                     * エラーが発生したインデックスを返します。
                     * 
                     * @return  エラーが発生したインデックス。
                     * @throws   java.lang.UnsupportedOperationException
                     *      サポートされない場合。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the index number where the exception is occured.
                     * 
                     * @return  the index number where the exception is occured.
                     * @throws   java.lang.UnsupportedOperationException
                     *      if the method is not suported.
                     * @since   notavaCC 1.0
                     */
                #}
                public final int getIndex() throws java.lang.UnsupportedOperationException {
                    if (index == -1)
                        throw new java.lang.UnsupportedOperationException();
                    return index;
                }
                
                #if (manualLocale.equals("ja")) {
                    /**
                     * エラーが発生した行番号を返します。
                     * 
                     * @return  エラーが発生した行番号。
                     * @throws   java.lang.UnsupportedOperationException
                     *      サポートされない場合。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the line number where the exception is occured.
                     * 
                     * @return  the line number where the exception is occured.
                     * @throws   java.lang.UnsupportedOperationException
                     *      if the method is not suported.
                     * @since   notavaCC 1.0
                     */
                #}
                public final int getLine() throws java.lang.UnsupportedOperationException {
                    if (line == -1)
                        throw new java.lang.UnsupportedOperationException();
                    return line;
                }
                
                #if (manualLocale.equals("ja")) {
                    /**
                     * エラーが発生したカラム番号を返します。
                     * 
                     * @return  エラーが発生したカラム番号。
                     * @throws   java.lang.UnsupportedOperationException
                     *      サポートされない場合。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the column number where the exception is occured.
                     * 
                     * @return  the column number where the exception is occured.
                     * @throws   java.lang.UnsupportedOperationException
                     *      if the method is not suported.
                     * @since   notavaCC 1.0
                     */
                #}
                public final int getColumn() throws java.lang.UnsupportedOperationException {
                    if (column == -1)
                        throw new java.lang.UnsupportedOperationException();
                    return column;
                }
                
                #if (manualLocale.equals("ja")) {
                    /**
                     * エラーの位置をあらわす、GNU Compiler Collection のスタイルの文字列を得ます。
                     * <p>
                     * このメソッドは、<code>getSourceName()</code> がサポートされない場合、{@link java.lang.UnsupportedOperationException} を発生させます。そうでない場合、<code>getLine()</code>がサポートされなければ、<code>getSourceName()</code> を返します。そうでない場合、<code>getSourceName() + ":" + getLine()</code> を返します。
                     * 
                     * @return  エラーの位置をあらわす、GNU Compiler Collection のスタイルの文字列。
                     * @throws   java.lang.UnsupportedOperationException
                     *      サポートされない場合。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the GNU-Compiler-Collection-style string that describes the position of the error.
                     * <p>
                     * This method throws an UnsupportedOperationException if <code>getSourceName()</code> is not supported.
                     * Otherwise, it returns <code>getSourceName()</code> if <code>getLine()</code> is not supported.
                     * Otherwise, it returns <code>getSourceName() + ":" + getLine()</code>.
                     * 
                     * @return
                     *      the GCC-style string describing the position of the error.
                     * @throws   UnsupportedOperationException
                     *      getSourceName() was not suppoted.
                     * @since   notavaCC 1.0
                     */
                #}
                public final String getGCCStylePositionString() throws java.lang.UnsupportedOperationException {
                    if (line == -1) {
                        return getSourceName();
                    } else {
                        return getSourceName() + ":" + line;
                    }
                }
                
                #if (manualLocale.equals("ja")) {
                    /**
                     * ￢＜＞＜∪∪へ入力された文法が曖昧である場合に発生します。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Indicates the source of the notavaCC, which generates this Java program, has ambiguity.
                     * @since   notavaCC 1.0
                     */
                #}
                public static class AmbiguousGrammarError extends $AssertionError {
                    public AmbiguousGrammarError(String message) {
                        super(message);
                    }
                }
            } // class $ParseException
            
            
            #if (manualLocale.equals("ja")) {
                /**
                 * {@link $Parser.$Token} の列を生成するオブジェクト。
                 * 
                 * @since   notavaCC 1.0
                 */
            #} else {
                /**
                 * Generator of a sequence of {@link $Parser.$Token}s.
                 * 
                 * @since   notavaCC 1.0
                 */
            #}
            #reserve("LexicalAnalyzer", topTypeNamespace);
            public static abstract class $LexicalAnalyzer {
                #if (manualLocale.equals("ja")) {
                    /**
                     * シーケンスの次のトークンを返します。
                     * <p>
                     * もしトークンが無い場合、{@link $Parser.$Token#getSymbolID()} が {@link $Parser#EOF_TOKEN} であるようなトークンを返します。この時の {@link $Parser.$Token#getImage()} や {@link $Parser.$Token#getIndex()} などの戻り値は規定されません。
                     * 
                     * @return  次のトークン。
                     * @since   notavaCC 1.0
                     */
                #} else {
                    /**
                     * Returns the next token in the sequence.
                     * <p>
                     * If there is no tokens, the {@linkplain $Parser.$Token#getSymbolID() symbol ID} of the returned value should be {@link $Parser#EOF_TOKEN}, but the {@linkplain $Parser.$Token#getImage() image} or the {@linkplain $Parser.$Token#getIndex() index} etc. are not specified (may be {@linkplain java.lang.UnsupportedOperationException unsupported}).
                     * 
                     * @return  the next token in the sequence.
                     * @since   notavaCC 1.0
                     */
                #}
                public abstract $Token next() throws $ParseException;
            } // $LexicalAnalyzer
            
            
            #if (manualLocale.equals("ja")) {
                /**
                 * デフォルトの実装を保持する名前空間です。
                 * 
                 * @since   notavaCC 1.0
                 */
            #} else {
                /**
                 * A namespace for the default implementations.
                 * 
                 * @since   notavaCC 1.0
                 */
            #}
            #reserve("Default", topTypeNamespace);
            public static interface $Default {
                #{
                    #if (manualLocale.equals("ja")) {
                        /**
                         * {@link $Parser.$Node} の簡単な実装を提供します。
                         * 
                         * @since   notavaCC 1.0
                         */
                    #} else {
                        /**
                         * Provides a simple implementation of {@link $Parser.$Node}.
                         * 
                         * @since   notavaCC 1.0
                         */
                    #}
                    public static abstract class $Node implements $Parser.$Node {
                        #if (targetVersion >= 1.5) {
                            private static final java.util.List<$Parser.$Node> EMPTY_LIST = new java.util.ArrayList<$Parser.$Node>();
                        #}
                        
                        private java.util.List$<$Parser.$Node> childNodes;
                        #if (manualLocale.equals("ja")) {
                            /**
                             * インスタンスを構築します。
                             * <p>
                             * <code>childNodes</code> は、{@link #$getChildNodes()} の戻り値として、そのまま使われます。
                             * <code>childNodes</code> の親のノードは、{@link $Parser.$Node#$setParentNode} を用いてこのノードに設定されます。
                             * {@link #$getParentNode()} のデフォルトの戻り値は <code>null</code> です。
                             * 
                             * @param   childNodes
                             *            子のノードのリスト。
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * Constucts an instance.
                             * <p>
                             * The returned list of {@link #$getChildNodes()} is the <code>childNodes</code> rather than a copy of it.
                             * The parent node of <code>childNodes</code> is set to <code>this</code> by {@link $Parser.$Node#$setParentNode}.
                             * The default value of {@link #$getParentNode()} is <code>null</code>.
                             * 
                             * @param   childNodes
                             *            a list of child nodes.
                             * @since   notavaCC 1.0
                             */
                        #}
                        public $Node(java.util.List$<$Parser.$Node> childNodes) {
                            this.childNodes = childNodes;
                            
                            java.util.Iterator it = childNodes.iterator();
                            while (it.hasNext()) {
                                $Parser.$Node node = ($Parser.$Node) it.next();
                                node.$setParentNode(this);
                            }
                        }
                        public java.util.List$<$Parser.$Node> $getChildNodes() {
                            return this.childNodes;
                        }
                        #if (visitorSupport) {
                            
                            public final void $accept($Visitor visitor) {
                                visitor.visitAll(this);
                            }
                        #}
                        
                        private $Parser.$Node parentNode;
                        public $Parser.$Node $getParentNode() {
                            return parentNode;
                        }
                        public void $setParentNode($Parser.$Node parentNode) {
                            this.parentNode = parentNode;
                        }
                        
                        #if (manualLocale.equals("ja")) {
                            /**
                             * 子のノード <code>oldChild</code> を <code>newChild</code> で置き換えます。
                             * <p>
                             * このメソッドは、<code>newChild.$setParentNode(this)</code> と <code>oldChild.$setParentNode(null)</code> を呼び出し、
                             * <code>$getChildNodes()</code> の中の <code>oldChild</code> を <code>newChild</code> で置き換えます。
                            #if (javadocForManual) {
                                 * また、{@link $Parser.$Type#$label()} によって返されるノードも置き換えます。
                            #} else {
                                 * また、{@link $Parser.$Node} のサブインターフェイスで定義される、特定の子を返すメソッドによって返されるノードも置き換えます。
                            #}
                             * 
                             * @param   oldChild
                             *            交換される子。
                             * @param   newChild
                             *            <code>oldChild</code> を置き換える新しい子。
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * Replaces the child node <code>oldChild</code> with <code>newChild</code>.
                             * <p>
                             * This method calls <code>newChild.$setParentNode(this)</code> and <code>oldChild.$setParentNode(null)</code>,
                             * and replaces <code>oldChild</code> in <code>$getChildNodes()</code> with <code>newChild</code>.
                            #if (javadocForManual) {
                                 * Also, this method replaces the values returned by {@link $Parser.$Type#$label()}.
                            #} else {
                                 * Also, this method replaces the values returned by the methods that are defined in the subinterfaces of {@link $Parser.$Node} and that return particular children.
                            #}
                             * 
                             * @param   oldChild
                             *            a replaced child node.
                             * @param   newChild
                             *            a child node to replace <code>oldChild</code>.
                             * @since   notavaCC 1.0
                             */
                        #}
                        #reserve("replaceChild", nodeMethodNamespace);
                        protected void $replaceChild($Parser.$Node oldChild, $Parser.$Node newChild) {
                            if (oldChild != null)
                                oldChild.$setParentNode(null);
                            $replaceAll($getChildNodes(), oldChild, newChild);
                            if (newChild != null)
                                newChild.$setParentNode(this);
                        }
                        
                        #if (manualLocale.equals("ja")) {
                            /**
                             * <code>oldNode</code> を、このノードで置き換えます。
                             * <p>
                             * このメソッドは、<code>(($Parser.$Default.$Node) oldNode.getParentNode()).$replaceChild(oldNode, this)</code> を呼び出します。
                             * 
                             * @param   oldNode
                             *            交換される子。
                             * @throws  java.lang.IllegalStateException
                             *            <code>oldNode.getParentNode() == null || !(oldNode.getParentNode() instanceof $Default.$Node)</code> の場合
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * Replaces <code>oldNode</code> with <code>this</code> node.
                             * <p>
                             * The method calls <code>(($Parser.$Default.$Node) oldNode.getParentNode()).$replaceChild(oldNode, this)</code>.
                             * 
                             * @param   oldNode
                             *            a node to replace.
                             * @throws  java.lang.IllegalStateException
                             *            if <code>oldNode.getParentNode() == null || !(oldNode.getParentNode() instanceof $Default.$Node)</code>
                             * @since   notavaCC 1.0
                             */
                        #}
                        #reserve("replace", nodeMethodNamespace);
                        public void $replace($Parser.$Node oldNode) throws java.lang.IllegalStateException {
                            $Parser.$Node newNode = this;
                            
                            $Parser.$Node parent = oldNode.$getParentNode();
                            if (parent == null || !(parent instanceof $Default.$Node))
                                throw new java.lang.IllegalStateException("oldNode.$getParentNode() should be a $Default.$Node.");
                            
                            $Default.$Node p = ($Default.$Node) parent;
                            p.$replaceChild(oldNode, newNode);
                        }
                        
                        public java.lang.String toString() {
                            java.io.StringWriter buffer = new java.io.StringWriter();
                            java.io.PrintWriter writer = new java.io.PrintWriter(buffer);
                            
                            $Parser.$Node node = this;
                            while (!(node instanceof $Parser.$Token)) {
                                if (node.getChildNodes().isEmpty()) {
                                    node = null;
                                    break;
                                }
                                node = ($Parser.$Node) node.getChildNodes().get(0);
                            }
                            if (node != null) {
                                $Parser.$Token token = ($Parser.$Token) node;
                                try {
                                    writer.print(token.getSourceName() + ":");
                                    writer.print(token.getLine() + ":");
                                    writer.println(token.getOriginalImage());
                                } catch (UnsupportedOperationException x) {
                                }
                            }
                            
                            toString(this, 0, writer);
                            writer.close();
                            if (writer.checkError())
                                throw new RuntimeException();
                            return buffer.toString();
                        }
                        private static void toString($Parser.$Node node, int indent, java.io.PrintWriter writer) {
                            for (int i = 0; i < indent; i++)
                                writer.print("  ");
                            
                            if (node instanceof $Token) {
                                $Token token = ($Token) node;
                                writer.println("token: " + token.getOriginalImage());
                            } else {
                                java.lang.String binaryName = node.getClass().getName();
                                binaryName = binaryName.substring(binaryName.lastIndexOf(".") + 1);
                                binaryName = binaryName.substring(binaryName.lastIndexOf("$") + 1);
                                writer.println(binaryName);
                                indent++;
                                java.util.Iterator it = node.$getChildNodes().iterator();
                                while (it.hasNext()) {
                                    $Parser.$Node n = ($Parser.$Node) it.next();
                                    toString(n, indent, writer);
                                }
                            }
                        }
                    } // $Default.$Node
                    
                    #if (manualLocale.equals("ja")) {
                        /**
                         * {@link $Parser.$Token} の簡単な実装を提供します。
                         * 
                         * @since   notavaCC 1.0
                         */
                    #} else {
                        /**
                         * Provides a simple implementation of {@link $Parser.$Token}.
                         * 
                         * @since   notavaCC 1.0
                         */
                    #}
                    public static class $Token extends $Default.$Node implements $Parser.$Token {
                        private final int symbolID;
                        private final java.lang.String image;
                        private final java.lang.String originalImage;
                        private final java.lang.String sourceName;
                        private final int index;
                        private final int line;
                        private final int column;
                        #if (manualLocale.equals("ja")) {
                            /**
                             * トークンを構築します。
                             * <p>
                             * このコンストラクタで構築したトークンでは、<code>getSourceName</code>、<code>getIndex</code>、<code>getLine</code>、<code>getColumn</code> はサポートされません。
                             * 
                             * @param   symbolID
                             *      <code>getSymbolID</code> と <code>getOriginalImage</code> で返される値。
                             * @param   image
                             *      <code>getImage</code> で返される値。
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * Constructs a token.
                             * <p>
                             * 
                             * @param   symbolID
                             *      the symbol ID of the token, returned by <code>getSymbolID</code> and <code>getOriginalImage</code>.
                             * @param   image
                             *      the image string, returned by <code>getImage</code>.
                             * @since   notavaCC 1.0
                             */
                        #}
                        public $Token(int symbolID, java.lang.String image) {
                            this(symbolID, image, image, null, -1, -1, -1);
                        }
                        
                        #if (manualLocale.equals("ja")) {
                            /**
                             * トークンを構築します。
                             * <p>
                             * 
                             * @param   symbolID
                             *      <code>getSymbolID</code> で返される値。
                             * @param   image
                             *      <code>getImage</code> で返される値。
                             * @param   originalImage
                             *      <code>getOriginalImage</code> で返される値。
                             * @param   sourceName
                             *      <code>getSourceName</code> で返される値。<code>getSourceName</code> がサポートされない場合 <code>null</code>。
                             * @param   index
                             *      <code>getIndex</code> で返される値。<code>getIndex</code> がサポートされない場合 <code>-1</code>。
                             * @param   line
                             *      <code>getLine</code> で返される値。<code>getLine</code> がサポートされない場合 <code>-1</code>。
                             * @param   column
                             *      <code>getColumn</code> で返される値。<code>getColumn</code> がサポートされない場合 <code>-1</code>。
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * Constructs a token.
                             * <p>
                             * 
                             * @param   symbolID
                             *      the symbol ID of the token, returned by <code>getSymbolID</code>.
                             * @param   image
                             *      the image string, returned by <code>getImage</code>.
                             * @param   originalImage
                             *      the image string, returned by <code>getOriginalImage</code>.
                             * @param   sourceName
                             *      the source name returned by <code>getSourceName</code>, or <code>null</code> if not supported.
                             * @param   index
                             *      the index returned by <code>getIndex</code>, or <code>-1</code> if not supported.
                             * @param   line
                             *      the line number returned by <code>getLine</code>, or <code>-1</code> if not supported.
                             * @param   column
                             *      the column number returned by <code>getColumn</code>, or <code>-1</code> if not supported.
                             * @since   notavaCC 1.0
                             */
                        #}
                        public $Token(int symbolID, java.lang.String image, java.lang.String originalImage, java.lang.String sourceName, int index, int line, int column) {
                            #if (targetVersion >= 1.5)
                                super($Parser.$Default.$Node.EMPTY_LIST);
                            #else
                                super(java.util.Collections.EMPTY_LIST);
                            this.symbolID = symbolID;
                            this.image = image;
                            this.originalImage = originalImage;
                            this.sourceName = sourceName;
                            this.index = index;
                            this.line = line;
                            this.column = column;
                        }
                        #if (manualLocale.equals("ja")) {
                            /**
                             * トークンを構築します。
                             * <p>
                             * 
                             * @param   symbolID
                             *      <code>getSymbolID</code> で返される値。
                             * @param   image
                             *      <code>getImage</code> で返される値。
                             * @param   originalImage
                             *      <code>getOriginalImage</code> で返される値。
                             * @param   token
                             *      <code>getSourceName</code>、<code>getIndex</code>、<code>getLine</code>、<code>getColumn</code> で返される値を与えるトークン。
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * Constructs a token.
                             * <p>
                             * 
                             * @param   symbolID
                             *      the symbol ID of the token, returned by <code>getSymbolID</code>.
                             * @param   image
                             *      the image string, returned by <code>getImage</code>.
                             * @param   originalImage
                             *      the image string, returned by <code>getOriginalImage</code>.
                             * @param   token
                             *      a token that gives the returned value of <code>getSourceName</code>, <code>getIndex</code>, <code>getLine</code> and <code>getColumn</code>.
                             * @since   notavaCC 1.0
                             */
                        #}
                        public $Token(int symbolID, java.lang.String image, java.lang.String originalImage, $Parser.$Token token) {
                            #if (targetVersion >= 1.5)
                                super($Parser.$Default.$Node.EMPTY_LIST);
                            #else
                                super(java.util.Collections.EMPTY_LIST);
                            String sourceName = null;
                            int index = -1;
                            int line = -1;
                            int column = -1;
                            try {
                                sourceName = token.getSourceName();
                            } catch (UnsupportedOperationException x) {
                            }
                            try {
                                index = token.getIndex();
                            } catch (UnsupportedOperationException x) {
                            }
                            try {
                                line = token.getLine();
                            } catch (UnsupportedOperationException x) {
                            }
                            try {
                                column = token.getColumn();
                            } catch (UnsupportedOperationException x) {
                            }
                            
                            this.symbolID = symbolID;
                            this.image = image;
                            this.originalImage = originalImage;
                            this.sourceName = sourceName;
                            this.index = index;
                            this.line = line;
                            this.column = column;
                        }
                        public int getSymbolID() {
                            return symbolID;
                        }
                        public boolean isWhite() {
                            return >>[[ tables.firstIndexOfWhiteTerminals() ]]<< <= symbolID && symbolID < >>[[ tables.superiorIndexOfWhiteTerminals() ]]<<;
                        }
                        public java.lang.String getImage() {
                            return image;
                        }
                        public java.lang.String getOriginalImage() {
                            return originalImage;
                        }
                        
                        
                        public java.lang.String getSourceName() throws java.lang.UnsupportedOperationException {
                            if (sourceName == null)
                                throw new java.lang.UnsupportedOperationException();
                            return sourceName;
                        }
                        public int getIndex() throws java.lang.UnsupportedOperationException {
                            if (index == -1)
                                throw new java.lang.UnsupportedOperationException();
                            return index;
                        }
                        public int getLine() throws java.lang.UnsupportedOperationException {
                            if (line == -1)
                                throw new java.lang.UnsupportedOperationException();
                            return line;
                        }
                        public int getColumn() throws java.lang.UnsupportedOperationException {
                            if (column == -1)
                                throw new java.lang.UnsupportedOperationException();
                            return column;
                        }
                        
                        public java.lang.String toString() {
                            java.lang.StringBuffer result = new java.lang.StringBuffer();
                            result.append("'");
                            result.append(getOriginalImage());
                            result.append("'");
                            
                            if (column != -1 || line != -1) {
                                result.append(" (");
                                if (line != -1) {
                                    result.append("line ");
                                    result.append(getLine());
                                    if (column != -1)
                                        result.append(", ");
                                }
                                if (column != -1) {
                                    result.append("column ");
                                    result.append(getColumn());
                                }
                                result.append(")");
                            }
                            return result.toString();
                        }
                    } // class $Default.$Token
                    
                    #if (manualLocale.equals("ja")) {
                        /**
                         * デフォルトの {@link $Parser.$LexicalAnalyzer}。
                         * 
                         * @since   notavaCC 1.0
                         */
                    #} else {
                        /**
                         * The default {@link $Parser.$LexicalAnalyzer}.
                         * 
                         * @since   notavaCC 1.0
                         */
                    #}
                    public static class $LexicalAnalyzer extends $Parser.$LexicalAnalyzer {
                        #if (manualLocale.equals("ja")) {
                            /**
                             * コンストラクタで渡された <code>sourceName</code>。
                             * 
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * The <code>sourceName</code> given to the constructor.
                             * 
                             * @since   notavaCC 1.0
                             */
                        #}
                        protected final java.lang.String sourceName;
                        #if (manualLocale.equals("ja")) {
                            /**
                             * コンストラクタで渡された <code>text</code>。
                             * 
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * The <code>text</code> given to the constructor.
                             * 
                             * @since   notavaCC 1.0
                             */
                        #}
                        protected final $CharSequence text;
                        #if (manualLocale.equals("ja")) {
                            /**
                             * {@link #text} の字句解析の対象範囲の終端のインデックス（exclusive）。
                             * 
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * The exclusive end index of the target {@link #text} region.
                             * 
                             * @since   notavaCC 1.0
                             */
                        #}
                        protected final int end;
                        #if (manualLocale.equals("ja")) {
                            /**
                             * コンストラクタで渡された <code>tabStop</code>。
                             * 
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * The <code>tabStop</code> given to the constructor.
                             * 
                             * @since   notavaCC 1.0
                             */
                        #}
                        protected final int tabStop;
                        #if (manualLocale.equals("ja")) {
                            /**
                             * コンストラクタへ渡された <code>table</code>。
                             * 
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * The <code>table</code> given to the constructor.
                             * 
                             * @since   notavaCC 1.0
                             */
                        #}
                        private final int[][] table;
                        #if (manualLocale.equals("ja")) {
                            /**
                             * コンストラクタへ渡された <code>initialState</code>。
                             * 
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * The <code>initialState</code> given to the constructor.
                             * 
                             * @since   notavaCC 1.0
                             */
                        #}
                        private final int initialState;
                        #if (manualLocale.equals("ja")) {
                            /**
                             * {@linkplain #next() 次}のトークンが始まるインデックス（zero based）。
                             * 
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * The zero-based index the {@linkplain #next() next} token begins at.
                             * 
                             * @since   notavaCC 1.0
                             */
                        #}
                        protected int index;
                        #if (manualLocale.equals("ja")) {
                            /**
                             * {@linkplain #next() 次}のトークンが始まる行番号（zero based）。
                             * 
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * The zero-based line number the {@linkplain #next() next} token begins at.
                             * 
                             * @since   notavaCC 1.0
                             */
                        #}
                        protected int line;
                        #if (manualLocale.equals("ja")) {
                            /**
                             * {@linkplain #next() 次}のトークンが始まるカラム番号（zero based）。
                             * 
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * The zero-based column number the {@linkplain #next() next} token begins at.
                             * 
                             * @since   notavaCC 1.0
                             */
                        #}
                        protected int column;
                        #if (manualLocale.equals("ja")) {
                            /**
                             * 字句解析器を構築します。
                             * <p>
                             * 
                             * @param   sourceName
                             *      解析すべきテキストの名前。これは、{@link $Parser.$Token#getSourceName()} の戻り値を設定するためだけに使用されます。
                             * @param   text
                             *      解析すべきテキスト
                             * @param   tabStop
                             *      テキスト中で、タブ文字に対して何文字ごとの位置に移動すべきか。これは、{@link $Parser.$Token#getColumn()} の戻り値を設定するためだけに使用されます。
                             * @since    notavaCC 1.0
                             */
                        #} else {
                            /**
                             * Constructs a lexical analyzer.
                             * <p>
                             * 
                             * @param   sourceName
                             *      the name of the source, used only to set the result of {@link $Parser.$Token#getSourceName()}.
                             * @param   text
                             *      the text to parse.
                             * @param   tabStop
                             *      the number that the tab character advances the cursor to the next column that is a multiple of.  <code>tabStop</code> is used only to set the result of {@link $Parser.$Token#getColumn()}.
                             * @since    notavaCC 1.0
                             */
                        #}
                        public $LexicalAnalyzer(java.lang.String sourceName, $CharSequence text, int tabStop) {
                            this(sourceName, text, tabStop, 0, text.length(), 0, 0);
                        }
                        #if (manualLocale.equals("ja")) {
                            /**
                             * テキストの一部分を解析する字句解析器を構築します。
                             * <p>
                             * 
                             * @param   sourceName
                             *      解析すべきテキストの名前。これは、{@link $Parser.$Token#getSourceName()} の戻り値を設定するためだけに使用されます。
                             * @param   text
                             *      解析すべきテキスト
                             * @param   tabStop
                             *      テキスト中で、タブ文字に対して何文字ごとの位置に移動すべきか。これは、{@link $Parser.$Token#getColumn()} の戻り値を設定するためだけに使用されます。
                             * @param   start
                             *      解析対象の領域の開始位置（inclusive）。
                             * @param   end
                             *      解析対象の領域の終了位置（exclusive）。
                             * @param   line
                             *      開始位置の行番号（zero-based）。
                             * @param   column
                             *      開始位置のカラム番号（zero-based）。
                             * @since    notavaCC 1.0
                             */
                        #} else {
                            /**
                             * Constructs a lexical analyzer that analyze a part of a text.
                             * <p>
                             * 
                             * @param   sourceName
                             *      the name of the source, used only to set the result of {@link $Parser.$Token#getSourceName()}.
                             * @param   text
                             *      the text to parse.
                             * @param   tabStop
                             *      the number that the tab character advances the cursor to the next column that is a multiple of.  <code>tabStop</code> is used only to set the result of {@link $Parser.$Token#getColumn()}.
                             * @param   start
                             *      the inclusive start index of the text region to analyze.
                             * @param   end
                             *      the exclusive end index of the text region to analyze.
                             * @param   line
                             *      the zero-based line number of the <code>start</code> index.
                             * @param   column
                             *      the zero-based column number of the <code>start</code> index.
                             * @since    notavaCC 1.0
                             */
                        #}
                        public $LexicalAnalyzer(java.lang.String sourceName, $CharSequence text, int tabStop, int start, int end, int line, int column) {
                            this(sourceName, text, tabStop, start, end, line, column, $Parser.$dfaTable, >>[[ tables.getIndexOfState(dfa.initialState()) ]]<<);
                        }
                        private $LexicalAnalyzer(java.lang.String sourceName, $CharSequence text, int tabStop, int start, int end, int line, int column, int[][] table, int initialState) {
                            this.sourceName = sourceName;
                            this.text = text;
                            this.end = end;
                            this.tabStop = tabStop;
                            this.table = table;
                            this.initialState = initialState;
                            this.index = start;
                            this.line = line;
                            this.column = column;
                        }
                        public $Parser.$Token next() throws $ParseException {
                            if (index >= end)
                                return new $Parser.$Default.$Token(EOF_TOKEN, "", "", sourceName, index, -1, -1);
                            
                            int state = initialState;
                            
                            final int startIndex = index;
                            final int startLine = line;
                            final int startColumn = column;
                            int lastHitID = -1;
                            int lastHitIndex = -1;
                            int lastHitLine = -1;
                            int lastHitColumn = -1;
                            int lastHitImageLength = 0;
                            StringBuffer image = new StringBuffer();
                            
                            try {
                            dfa:
                                while (index < end) {
                                    char ch = nextChar();
                                    image.append(ch);
                                    
                                    int[] map = table[state];
                                    int base = 1;
                                    int width = (map.length - 1) / 3;
                                    #java.lang.String dfaLinearSearchThreshold = environment.parameters.getProperty("JavaCodeGenerator.dfaLinearSearchThreshold", "3");
                                    while (width > >>[[ dfaLinearSearchThreshold ]]<<) {
                                        int halfWidth = width >> 1;
                                        int mid = base + halfWidth * 3;
                                        if (ch >= map[mid]) {
                                            base = mid;
                                            width -= halfWidth;
                                        } else {
                                            width = halfWidth;
                                        }
                                    }
                                    int i = 0;
                                    for (;;) {
                                        if (i++ >= width)
                                            break dfa;
                                        if (map[base] <= ch & ch <= map[base + 1]) {
                                            state = map[base + 2];
                                            break;
                                        }
                                        base += 3;
                                    }
                                    int hitID = table[state][0];
                                    if (hitID != -1) {
                                        lastHitID = hitID;
                                        lastHitIndex = index;
                                        lastHitLine = line;
                                        lastHitColumn = column;
                                        lastHitImageLength = image.length();
                                    }
                                }
                                
                                if (lastHitID == -1) {
                                    throw new $ParseException("An invalid character sequence starting with '" + text.$subSequence(startIndex, index) + "' (line " + (startLine + 1) + ", column " + (startColumn + 1) + ")", sourceName, startIndex, startLine + 1, startColumn + 1);
                                }
                            } finally {
                                index = startIndex;
                                line = startLine;
                                column = startColumn;
                            }
                            
                            $Parser.$Token result = new $Parser.$Default.$Token(lastHitID, image.substring(0, lastHitImageLength).intern(), text.$subSequence(startIndex, lastHitIndex).toString().intern(), sourceName, startIndex, startLine + 1, startColumn + 1);
                            
                            index = lastHitIndex;
                            line = lastHitLine;
                            column = lastHitColumn;
                            
                            return result;
                        } // next()
                        #if (manualLocale.equals("ja")) {
                            /**
                             * {@link #text} 中の次の文字を返します。
                             * <p>
                             * このメソッドは、{@link #index} から始まる１文字を返します。このメソッドは {@link #index}、{@link #line}、{@link #column} を、次の文字をさすように更新しなければなりません。このメソッドは、ユニコード・エスケープのように、文字列を文字として扱いたい場合にオーバーライドされます。
                             * 
                             * @return  次の文字。
                             * @throws  java.lang.IndexOutOfBoundsException
                             *          <code>index >= end</code> の場合。
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * Returns the next character in the {@link #text}.
                             * <p>
                             * This method returns the character starting at {@link #index} in the sequence and updates {@link #index}, {@link #line}, and {@link #column} to point the next character of the returned character.  This method is overridden in order to deal with a string as a character through the input sequence like Unicode escapes of Java.
                             * 
                             * @return  the next character in the sequence.
                             * @throws  java.lang.IndexOutOfBoundsException  if <code>index >= end</code>
                             * @since   notavaCC 1.0
                             */
                        #}
                        protected char nextChar() throws $ParseException, java.lang.IndexOutOfBoundsException {
                            return nextExactChar();
                        }
                        #if (manualLocale.equals("ja")) {
                            /**
                             * ユニコード・エスケープをふまえて{@link #text} 中の次の文字を返します。
                             * 
                             * @return  次の文字。
                             * @throws  ParseException
                             *              無効なユニコード・エスケープが存在した場合。
                             * @throws  java.lang.IndexOutOfBoundsException
                             *              <code>index >= end</code> の場合。
                             * @since   notavaCC 1.0
                             */
                        #} else {
                            /**
                             * Returns the next character in the {@link #text} based on the Unicode escapes.
                             * 
                             * @return  the next character in the sequence.
                             * @throws  ParseException
                             *              for a illegal escape.
                             * @throws  java.lang.IndexOutOfBoundsException
                             *              if <code>index >= end</code>
                             * @since   notavaCC 1.0
                             */
                        #}
                        protected final char nextUnicodeEscapedChar() throws ParseException, java.lang.IndexOutOfBoundsException {
                            int startIndex = this.index;
                            int startLine = line;
                            int startColumn = column;
                            {
                                int index = startIndex;
                                if (text.charAt(index) == '\\' && index + 1 < end && text.charAt(index + 1) == 'u') {
                                    int count = 0;
                                    while (index >= 0 && text.charAt(index--) == '\\')
                                        count++;
                                    if (count % 2 == 0)
                                        return nextExactChar();
                                } else {
                                    return nextExactChar();
                                }
                            }
                            
                            char ch = nextExactChar();
                            $assert (ch =='\\');
                            ch = nextExactChar();
                            $assert (ch == 'u');
                            try {
                                do {
                                    ch = nextExactChar();
                                } while (ch == 'u');
                                StringBuffer number = new StringBuffer(4);
                                number.append(ch);
                                number.append(nextExactChar());
                                number.append(nextExactChar());
                                number.append(nextExactChar());
                                int code = Integer.parseInt(number.toString(), 16);
                                return (char) code;
                            } catch (NumberFormatException x) {
                            } catch (IndexOutOfBoundsException x) {
                            }
                            throw new ParseException("illigal Unicode escape: " + text.$subSequence(startIndex, index), sourceName, startIndex, startLine, startColumn);
                        }
                        private final char nextExactChar() throws $ParseException, java.lang.IndexOutOfBoundsException {
                            if (index >= end)
                                throw new java.lang.IndexOutOfBoundsException();
                            char ch = text.charAt(index++);
                            switch (ch) {
                            case '\r':
                                if (index < end && text.charAt(index) == '\n')
                                    break;
                            case '\n':
                                line++;
                                column = 0;
                                break;
                            case '\t':
                                if (tabStop >= 1)
                                    column = column - (column % tabStop) + tabStop;
                                break;
                            default:
                                column++;
                                break;
                            }
                            return ch;
                        }
                    } // class $Default.$LexicalAnalyzer
                    
                    
                    #// user-defined types
                    #Set defaultClasses = new HashSet();
                    #Iterator outer = tables.userDefinedTypes().iterator();
                    #while (outer.hasNext()) {
                        #Type type = (Type) outer.next();
                        #TypeNonterminal nonterminal = ((UserDefinedType) type).getTypeNonterminal();
                        #if (nonterminal != null) {
                            #for (;;) {
                                #if (defaultClasses.add(type)) {
                                    #type = (Type) type.getDirectSuperTypes().iterator().next();
                                    #if (type instanceof UserDefinedType)
                                        #continue;
                                #}
                                #break;
                            #}
                        #}
                    #}
                    #outer = tables.userDefinedTypes().iterator();
                    #while (outer.hasNext()) {
                        #UserDefinedType type = (UserDefinedType) outer.next();
                        #if (defaultClasses.contains(type)) {
                            #boolean isAbstract = (type.getTypeNonterminal() == null);
                            #java.lang.String identifier = type.getName();
                            #Type superType = (Type) type.getDirectSuperTypes().iterator().next();
                            #String superTypeString;
                            #if (superType instanceof UserDefinedType) {
                                #superTypeString = ((UserDefinedType) superType).getName();
                            #} else {
                                #superTypeString = "$Node";
                            #}
                            #if (manualLocale.equals("ja")) {
                                /**
                                 * <code>>>[[ encodeForHTML(identifier) ]]<<</code> の実装。
                                #if (javadocForManual) {
                                     * <p>
                                     * このクラスは、<i><a href="../input.html#syntax:TypeDefinition" class="symbol">TypeDefinition</a></i> で定義される、<code>$abstract</code>ではない非終端記号 $Type に対して出力されます。また、このクラスを出力させる <i><a href="../input.html#syntax:TypeDefinition" class="symbol">TypeDefinition</a></i> の <i><a href="../input.html#syntaxref:TypeDefinition:supertypes" class="symbol">supertypes</a></i> に列挙される最初の <i><a href="../input.html#syntaxref:supertypes:TypeName" class="symbol">TypeName</a></i> に対しても同様のクラスが出力されます。
                                     * <p>
                                     * このクラスは、<code>$abstract</code> な <i><a href="../input.html#syntax:TypeDefinition" class="symbol">TypeDefinition</a></i> に対しては <code>abstract</code> になります。そうでない場合、<code>abstract</code> ではなくなります。
                                     * <p>
                                     * このクラスは、<i><a href="../input.html#syntaxref:TypeDefinition:supertypes" class="symbol">supertypes</a></i> が存在しない場合 {@link $Parser.$Default.$Node} のサブクラスになります。そうでない場合、<i><a href="../input.html#syntaxref:TypeDefinition:supertypes" class="symbol">supertypes</a></i> の最初の <i><a href="../input.html#syntaxref:supertypes:TypeName" class="symbol">TypeName</a></i> と同じ名前の {@link $Parser.$Default} のネストしたクラスのサブクラスになります。
                                #}
                                 * 
                                 * @since   notavaCC 1.0
                                 */
                            #} else {
                                /**
                                 * An implementation of <code>>>[[ encodeForHTML(identifier) ]]<<</code> node.
                                 * 
                                 * @since   notavaCC 1.0
                                 */
                            #}
                            public >>[[ isAbstract ? "abstract " : "" ]]<<static class >>[[ identifier ]]<< extends $Default.>>[[ superTypeString ]]<< implements >>[[ getTypeName(type) ]]<< {
                                #Iterator it7 = type.getMapNameToField().values().iterator();
                                #while (it7.hasNext()) {
                                    #Field field = (Field) it7.next();
                                    #Type fieldType = field.getType();
                                    #Field superField = superType.getField(field.getName());
                                    #if (superField == null) {
                                        #if (fieldType instanceof ArrayType) {
                                            #ArrayType arrayType = (ArrayType) fieldType;
                                            private >>[[ getTypeName(fieldType) ]]<< >>[[ field.getName() ]]<< = new java.util.LinkedList$<>>[[ getTypeName(arrayType.getComponentType()) ]]<<>();
                                        #} else {
                                            private >>[[ getTypeName(fieldType) ]]<< >>[[ field.getName() ]]<< = null;
                                        #}
                                        public >>[[ getTypeName(fieldType) ]]<< >>[[ field.getName() ]]<<() {
                                            return >>[[ field.getName() ]]<<;
                                        }
                                    #} else if (!fieldType.equals(superField.getType())) {
                                        public >>[[ getTypeName(fieldType) ]]<< >>[[ field.getName() ]]<<() {
                                            return (>>[[ getTypeName(fieldType) ]]<<) super.>>[[ field.getName() ]]<<();
                                        }
                                    #}
                                #}
                                
                                #if (manualLocale.equals("ja")) {
                                    /**
                                    #if (javadocForManual) {
                                         * <code>source.$getChildNodes()</code> や <code>source.{@linkplain $Parser.$Type#$label $label()}</code> などによって与えられる子を持つようにノードを構築します。リストオブジェクトはコピーされます。
                                    #} else {
                                         * <code>source.$getChildNodes()</code> などによって与えられる子を持つようにノードを構築します。リストオブジェクトはコピーされます。
                                    #}
                                     * 
                                     * @param   source
                                     *            ソース。
                                     */
                                #} else {
                                    /**
                                    #if (javadocForManual) {
                                         * Constructs a instance with the child nodes given by <code>source.$getChildNodes()</code> and <code>source.{@linkplain $Parser.$Type#$label $label()}</code>.  <code>List</code> objects are copied.
                                    #} else {
                                         * Constructs a instance with the child nodes given by <code>source.$getChildNodes()</code> etc.  <code>List</code> objects are copied.
                                    #}
                                     * 
                                     * @param   source
                                     *            the source.
                                     */
                                #}
                                public >>[[ identifier ]]<<(>>[[ getTypeName(type) ]]<< source) {
                                    #if (superType instanceof UserDefinedType) {
                                        super(($Parser.>>[[ superTypeString ]]<<) source);
                                    #} else {
                                        super(new java.util.LinkedList$<$Parser.$Node>(source.$getChildNodes()));
                                    #}
                                    
                                    #Iterator it8 = type.getMapNameToField().values().iterator();
                                    #while (it8.hasNext()) {
                                        #Field field = (Field) it8.next();
                                        #if (superType.getField(field.getName()) == null) {
                                            #Type fieldType = field.getType();
                                            #if (fieldType instanceof ArrayType) {
                                                #ArrayType arrayType = (ArrayType) fieldType;
                                                this.>>[[ field.getName() ]]<< = new java.util.LinkedList$<>>[[ getTypeName(arrayType.getComponentType()) ]]<<>(source.>>[[ field.getName() ]]<<());
                                            #} else {
                                                this.>>[[ field.getName() ]]<< = source.>>[[ field.getName() ]]<<();
                                            #}
                                        #}
                                    #}
                                }
                                
                                #if (manualLocale.equals("ja")) {
                                    /**
                                     * <code>parameters</code> で与えられる子を持つようにノードを構築します。
                                     * 
                                     * @param   parameters
                                     *            初期化パラメタ。
                                     * @param   compact
                                     *            <code>true</code> の場合、ラベル付けされていない子を除いてノードを作る。
                                     */
                                #} else {
                                    /**
                                     * Constructs a instance with the child nodes given by <code>parameters</code>.
                                     * 
                                     * @param   parameters
                                     *            the parameters for initialization.
                                     * @param   compact
                                     *            remove the children that are not labeled.
                                     */
                                #}
                                public >>[[ identifier ]]<<($Parser.$NodeInitializationParameters parameters, boolean compact) {
                                    #if (superType instanceof UserDefinedType) {
                                        super(parameters, compact);
                                    #} else {
                                        super(new java.util.LinkedList$<$Parser.$Node>());
                                    #}
                                    
                                    $Parser.$Node[] childNodes = parameters.childNodes;
                                    java.util.List labelsIDList = parameters.labelsIDList;
                                    
                                    java.util.Iterator it = labelsIDList.iterator();
                                    int size = labelsIDList.size();
                                    for (int i = 0; i < size; i++) {
                                        #if (labelWordCount == 1) {
                                            java.lang.Integer number = (java.lang.Integer) it.next();
                                            int no = number.intValue();
                                        #} else {
                                            java.math.BigInteger number = (java.math.BigInteger) it.next();
                                        #}
                                        #if (!(superType instanceof UserDefinedType)) {
                                            #if (labelWordCount == 1) {
                                                if (!compact || no != 0)
                                                    $getChildNodes().add(childNodes[i]);
                                            #} else {
                                                if (!compact || !number.equals(java.math.BigInteger.ZERO))
                                                    $getChildNodes().add(childNodes[i]);
                                            #}
                                            childNodes[i].$setParentNode(this);
                                        #}
                                        #Iterator it9 = type.getDeclaredFields().iterator();
                                        #while (it9.hasNext()) {
                                            #Field field = (Field) it9.next();
                                            #if (superType.getField(field.getName()) == null) {
                                                #java.lang.String fieldTypeName = getTypeName(field.getType());
                                                
                                                #int labelNo = tables.getIndexOfLabel(field.getName());
                                                #java.lang.String match;
                                                #if (labelWordCount == 1) {
                                                    #match = "(no & (1 << " + labelNo + ")) != 0";
                                                #} else {
                                                    #match = "number.testBit(" + labelNo + ")";
                                                #}
                                                if (>>[[ match ]]<<) {
                                                    #if (field.getType() instanceof ArrayType) {
                                                        #Type componentType = ((ArrayType) field.getType()).getComponentType();
                                                        this.>>[[ field.getName() ]]<<.add((>>[[ getTypeName(componentType) ]]<<) childNodes[i]);
                                                    #} else {
                                                        this.>>[[ field.getName() ]]<< = (>>[[ fieldTypeName ]]<<) childNodes[i];
                                                    #}
                                                }
                                            #}
                                        #}
                                    }
                                }
                                
                                protected void $replaceChild($Parser.$Node oldChild, $Parser.$Node newChild) {
                                    #Iterator it10 = type.getMapNameToField().values().iterator();
                                    #while (it10.hasNext()) {
                                        #Field field = (Field) it10.next();
                                        #if (superType.getField(field.getName()) == null) {
                                            #java.lang.String fieldTypeName = getTypeName(field.getType());
                                            #if (field.getType() instanceof ArrayType) {
                                                #Type componentType = ((ArrayType) field.getType()).getComponentType();
                                                if (this.>>[[ field.getName() ]]<<.contains(oldChild))
                                                    $replaceAll(this.>>[[ field.getName() ]]<<, oldChild, (>>[[ getTypeName(componentType) ]]<<) newChild);
                                            #} else {
                                                if (oldChild == null ? this.>>[[ field.getName() ]]<< == null : oldChild.equals(this.>>[[ field.getName() ]]<<))
                                                    this.>>[[ field.getName() ]]<< = (>>[[ fieldTypeName ]]<<) newChild;
                                            #}
                                        #}
                                    #}
                                    super.$replaceChild(oldChild, newChild);
                                }
                            } // class >>[[ identifier ]]<<
                        #}
                    #}
                    
                    
                #}
            } // $Default
            
            #{
                #if (graphStructuredStack) {
                    #allocate("GrpahStack", topTypeNamespace);
                    private static class $GrpahStack {
                        public static final $GrpahStack[] EMPTY_STACK = null;
                        
                        private $GrpahStack[] previous;
                        private java.lang.Object node;
                        private int state;
                        private $GrpahStack($GrpahStack[] previous, java.lang.Object node, int state) {
                            this.previous = previous;
                            this.node = node;
                            this.state = state;
                        }
                        public static $GrpahStack push($GrpahStack[] stack, java.lang.Object node, int state) {
                            return new $GrpahStack(stack, node, state);
                        }
                        public int topState() {
                            return state;
                        }
                        public java.lang.Object topNode() {
                            return node;
                        }
                        public static $GrpahStack[] pop($GrpahStack stack) {
                            return stack.previous;
                        }
                    }
                    #allocate("ConcreteSyntaxTree", topTypeNamespace);
                    private static class $ConcreteSyntaxTree {
                        public int symbolID;
                        public $ConcreteSyntaxTreeList children;
                        public $ConcreteSyntaxTree(int symbolID, $ConcreteSyntaxTreeList children) {
                            this.symbolID = symbolID;
                            this.children = children;
                        }
                    }
                    #allocate("ConcreteSyntaxTreeList", topTypeNamespace);
                    private static class $ConcreteSyntaxTreeList {
                        #if (labelWordCount == 1) {
                            public java.lang.Integer labels;
                        #} else {
                            public java.math.BigInteger labels;
                        #}
                        public java.lang.Object node;
                        public $ConcreteSyntaxTreeList next;
                        
                        #if (labelWordCount == 1) {
                            private $ConcreteSyntaxTreeList(java.lang.Integer labels, java.lang.Object node, $ConcreteSyntaxTreeList next) {
                                this.labels = labels;
                                this.node = node;
                                this.next = next;
                            }
                        #} else {
                            private $ConcreteSyntaxTreeList(java.math.BigInteger labels, java.lang.Object node, $ConcreteSyntaxTreeList next) {
                                this.labels = labels;
                                this.node = node;
                                this.next = next;
                            }
                        #}
                    }
                #} else {
                    #allocate("IntegerStack", topTypeNamespace);
                    private static class $IntegerStack {
                        public static final $IntegerStack EMPTY_STACK = new $IntegerStack(null, 0);
                        
                        private $IntegerStack previous;
                        private int value;
                        private $IntegerStack($IntegerStack previous, int value) {
                            this.previous = previous;
                            this.value = value;
                        }
                        public boolean isEmpty() {
                            return previous == null;
                        }
                        public static $IntegerStack push($IntegerStack stack, int value) {
                            return new $IntegerStack(stack, value);
                        }
                        public int top() {
                            return value;
                        }
                        public static $IntegerStack pop($IntegerStack stack) {
                            return stack.previous;
                        }
                    }
                    #allocate("State", topTypeNamespace);
                    private static class $State {
                        public $IntegerStack stateStack;
                        public $IntegerStack shiftReduceHistory;
                        public $State($IntegerStack stateStack, $IntegerStack shiftReduceHistory) {
                            this.stateStack = stateStack;
                            this.shiftReduceHistory = shiftReduceHistory;
                        }
                    }
                #}
                
                #if (tables.tags().length > 0) {
                    #reserve("taggedTokenShifted", topMethodNamespace);
                    protected void $taggedTokenShifted($Parser.$LexicalAnalyzer analyzer, $Parser.$Token token, int tag) {
                    }
                #}
                
                #allocate("parse", topMethodNamespace);
                private $Node $parse(int[][] lrTable, int[][] reductions, int[] tagTable, int initialState, int classSymbolMaximum, $LexicalAnalyzer analyzer) throws $ParseException {
                    final int labelWordCount = >>[[ labelWordCount ]]<<;
                    final int firstIndexOfWhiteTerminals = >>[[ tables.firstIndexOfWhiteTerminals() ]]<<;
                    final int superiorIndexOfWhiteTerminals = >>[[ tables.superiorIndexOfWhiteTerminals() ]]<<;
                    #if (graphStructuredStack) {
                        java.util.ArrayList$<$GrpahStack> states = new java.util.ArrayList$<$GrpahStack>();
                        {
                            // phase 1:  scan.
                            java.util.ArrayList$<$GrpahStack> nextStates = new java.util.ArrayList$<$GrpahStack>();
                            states.add($GrpahStack.push($GrpahStack.EMPTY_STACK, null, initialState));
                        tokenLoop:
                            for (;;) {
                                $Token nextToken;
                                int nextTokenID;
                                do {
                                    nextToken = analyzer.next();
                                    nextTokenID = nextToken.getSymbolID();
                                } while (firstIndexOfWhiteTerminals <= nextTokenID && nextTokenID < superiorIndexOfWhiteTerminals);
                                
                                int initialStatesSize = states.size();
                            reduceLoop:
                                for (int i = 0; i < states.size(); i++) {
                                    final $GrpahStack stack = ($GrpahStack) states.get(i);
                                    final int stateID = stack.topState();
                                    //System.out.print(stateID + ", ");
                                    int[] tokenIDToReductionID = lrTable[stateID * 3 + 2];
                                    int j = $binarySearch(tokenIDToReductionID, nextTokenID);
                                    for (; j < tokenIDToReductionID.length; j += 2) {
                                        if (tokenIDToReductionID[j] > nextTokenID) {
                                            break;
                                        } else if (tokenIDToReductionID[j] == nextTokenID) {
                                            final int reductionID = tokenIDToReductionID[j + 1];
                                            final int[] reduction = reductions[reductionID];
                                            
                                            $reduce(states, stack, reduction, reduction.length - 1, null, lrTable);
                                        }
                                    }
                                }
                                //System.out.println();
                                
                            shiftLoop:
                                for (int i = 0; i < states.size(); i++) {
                                    final $GrpahStack stack = ($GrpahStack) states.get(i);
                                    final int stateID = stack.topState();
                                    
                                    final int[] tokenIDToNextState = lrTable[stateID * 3 + 0];
                                    int k = $binarySearch(tokenIDToNextState, nextTokenID);
                                    for (; k < tokenIDToNextState.length; k += 2) {
                                        if (tokenIDToNextState[k] > nextTokenID) {
                                            break;
                                        } else if (tokenIDToNextState[k] == nextTokenID) {
                                            final int nextStateID = tokenIDToNextState[k + 1];
                                            nextStates.add($GrpahStack.push(new $GrpahStack[] { stack }, nextToken, nextStateID));
                                        }
                                    }
                                }
                                
                                if (nextStates.isEmpty()) {
                                    states.subList(initialStatesSize, states.size()).clear();
                                    $error(analyzer, lrTable, reductions, states, nextToken);
                                    continue tokenLoop;
                                }
                                
                                #if (tables.tags().length > 0) {
                                    for (int i = 0; i < nextStates.size(); i++) {
                                        $GrpahStack stack = ($GrpahStack) nextStates.get(i);
                                        int stateID = stack.topState();
                                        for (int j = 0; j < tagTable.length; j += 2) {
                                            if (tagTable[j] == stateID) {
                                                if (nextStates.size() > 1)
                                                    throw new $ParseException.AmbiguousGrammarError(>>[[ quoted(file.getName()) ]]<< + " has ambiguous tags.");
                                                else
                                                    $taggedTokenShifted(analyzer, nextToken, tagTable[j + 1]);
                                                break;
                                            }
                                        }
                                    }
                                #}
                                
                                if (nextTokenID == EOF_TOKEN) {
                                    states = nextStates;
                                    break tokenLoop;
                                }
                                
                                java.util.ArrayList$<$GrpahStack> tmp = states;
                                states = nextStates;
                                nextStates = tmp;
                                nextStates.clear();
                            }
                        }
                        if (states.size() != 1 || $GrpahStack.pop(($GrpahStack) states.get(0)).length != 1) {
                            throw new $ParseException.AmbiguousGrammarError(>>[[ quoted(file.getName()) ]]<< + " is ambiguous.");
                        }
                        
                        // phase 2:  build a tree.
                        $GrpahStack stack = $GrpahStack.pop(($GrpahStack) states.get(0))[0];
                        return ($Node) $createNode(($ConcreteSyntaxTree) stack.topNode(), classSymbolMaximum);
                    #} else {
                        java.util.LinkedList$<$Token> tokens = new java.util.LinkedList$<$Token>();
                        int SHIFT = -1;
                        java.util.ArrayList$<$State> states = new java.util.ArrayList$<$State>();
                        {
                            // phase 1:  scan.
                            java.util.ArrayList$<$State> nextStates = new java.util.ArrayList$<$State>();
                            states.add(new $State($IntegerStack.push($IntegerStack.EMPTY_STACK, initialState), $IntegerStack.EMPTY_STACK));
                        tokenLoop:
                            for (;;) {
                                $Token nextToken;
                                int nextTokenID;
                                do {
                                    nextToken = analyzer.next();
                                    nextTokenID = nextToken.getSymbolID();
                                    if (nextTokenID != EOF_TOKEN)
                                        tokens.addLast(nextToken);
                                } while (firstIndexOfWhiteTerminals <= nextTokenID && nextTokenID < superiorIndexOfWhiteTerminals);
                                
                                int initialStatesSize = states.size();
                            reduceLoop:
                                for (int i = 0; i < states.size(); i++) {
                                    final $State state = ($State) states.get(i);
                                    final int stateID = state.stateStack.top();
                                    //System.out.print(stateID + ", ");
                                    int[] tokenIDToReductionID = lrTable[stateID * 3 + 2];
                                    int j = $binarySearch(tokenIDToReductionID, nextTokenID);
                                    for (; j < tokenIDToReductionID.length; j += 2) {
                                        if (tokenIDToReductionID[j] > nextTokenID) {
                                            break;
                                        } else if (tokenIDToReductionID[j] == nextTokenID) {
                                            final int reductionID = tokenIDToReductionID[j + 1];
                                            final int[] reduction = reductions[reductionID];
                                            final int nextSymbolID = reduction[0];
                                            
                                            $IntegerStack stateStack = state.stateStack;
                                            for (int k = reduction.length - 1; k >= 1; k -= labelWordCount)
                                                stateStack = $IntegerStack.pop(stateStack);
                                            
                                            $IntegerStack shiftReduceHistory = state.shiftReduceHistory;
                                            shiftReduceHistory = $IntegerStack.push(shiftReduceHistory, reductionID);
                                            
                                            final int returnedStateID = stateStack.top();
                                            final int[] symbolIDToNextState = lrTable[returnedStateID * 3 + 1];
                                            int k = $binarySearch(symbolIDToNextState, nextSymbolID);
                                            for (; k < symbolIDToNextState.length; k += 2) {
                                                if (symbolIDToNextState[k] > nextSymbolID) {
                                                    break;
                                                } else if (symbolIDToNextState[k] == nextSymbolID) {
                                                    final int nextStateID = symbolIDToNextState[k + 1];
                                                    states.add(new $State($IntegerStack.push(stateStack, nextStateID), shiftReduceHistory));
                                                    break;
                                                }
                                            }
                                        }
                                    }
                                }
                                //System.out.println();
                                
                            shiftLoop:
                                for (int i = 0; i < states.size(); i++) {
                                    final $State state = ($State) states.get(i);
                                    final int stateID = state.stateStack.top();
                                    
                                    final int[] tokenIDToNextState = lrTable[stateID * 3 + 0];
                                    int k = $binarySearch(tokenIDToNextState, nextTokenID);
                                    for (; k < tokenIDToNextState.length; k += 2) {
                                        if (tokenIDToNextState[k] > nextTokenID) {
                                            break;
                                        } else if (tokenIDToNextState[k] == nextTokenID) {
                                            final int nextStateID = tokenIDToNextState[k + 1];
                                            $IntegerStack stateStack = state.stateStack;
                                            stateStack = $IntegerStack.push(stateStack, nextStateID);
                                            
                                            $IntegerStack shiftReduceHistory = state.shiftReduceHistory;
                                            shiftReduceHistory = $IntegerStack.push(shiftReduceHistory, SHIFT);
                                            
                                            nextStates.add(new $State(stateStack, shiftReduceHistory));
                                        }
                                    }
                                }
                                
                                if (nextStates.isEmpty()) {
                                    if (nextTokenID != EOF_TOKEN)
                                        tokens.removeLast();
                                    states.subList(initialStatesSize, states.size()).clear();
                                    $error(analyzer, lrTable, reductions, states, nextToken);
                                    continue tokenLoop;
                                }
                                
                                #if (tables.tags().length > 0) {
                                    for (int i = 0; i < nextStates.size(); i++) {
                                        $State state = ($State) nextStates.get(i);
                                        int stateID = state.stateStack.top();
                                        for (int j = 0; j < tagTable.length; j += 2) {
                                            if (tagTable[j] == stateID) {
                                                if (nextStates.size() > 1)
                                                    throw new $ParseException.AmbiguousGrammarError(>>[[ quoted(file.getName()) ]]<< + " has ambiguous tags.");
                                                else
                                                    $taggedTokenShifted(analyzer, nextToken, tagTable[j + 1]);
                                                break;
                                            }
                                        }
                                    }
                                #}
                                
                                if (nextTokenID == EOF_TOKEN) {
                                    states = nextStates;
                                    break tokenLoop;
                                }
                                
                                java.util.ArrayList$<$State> tmp = states;
                                states = nextStates;
                                nextStates = tmp;
                                nextStates.clear();
                            }
                        }
                        
                        if (states.size() != 1)
                            throw new $ParseException.AmbiguousGrammarError(>>[[ quoted(file.getName()) ]]<< + " is ambiguous.");
                        $State state = ($State) states.get(0);
                        
                        // phase 2:  build a tree.
                        {
                            $IntegerStack shiftReduceHistory = state.shiftReduceHistory;
                            {
                                shiftReduceHistory = $IntegerStack.pop(shiftReduceHistory); // remove the last shift for EOF.
                                $IntegerStack tracer = $IntegerStack.EMPTY_STACK;
                                while (!shiftReduceHistory.isEmpty()) {
                                    tracer = $IntegerStack.push(tracer, shiftReduceHistory.top());
                                    shiftReduceHistory = $IntegerStack.pop(shiftReduceHistory);
                                }
                                shiftReduceHistory = tracer;
                            }
                            #if (labelWordCount == 1) {
                                final java.lang.Integer ZERO = new java.lang.Integer(0);
                            #} else {
                                final java.math.BigInteger ZERO = java.math.BigInteger.ZERO;
                            #}
                            
                            java.util.Iterator it = tokens.iterator();
                            java.util.LinkedList$<java.lang.Object> stack = new java.util.LinkedList$<java.lang.Object>();
                        mainLoop:
                            while (!shiftReduceHistory.isEmpty()) {
                                int shiftReduce = shiftReduceHistory.top();
                                shiftReduceHistory = $IntegerStack.pop(shiftReduceHistory);
                                if (shiftReduce == SHIFT) {
                                    $Token token;
                                    do {
                                        token = ($Token) it.next();
                                        stack.addLast(token);
                                    } while (firstIndexOfWhiteTerminals <= token.getSymbolID() && token.getSymbolID() < superiorIndexOfWhiteTerminals);
                                } else {
                                    final int[] reduction = reductions[shiftReduce];
                                    boolean isFinal = shiftReduceHistory.isEmpty();
                                    if (isFinal) {
                                        while (it.hasNext())
                                            stack.addLast(it.next());
                                    }
                                    java.util.LinkedList$<java.lang.Object> list = new java.util.LinkedList$<java.lang.Object>();
                                    for (int j = reduction.length - 1; j >= 1; j -= labelWordCount) {
                                        java.lang.Object node;
                                        for (;;) {
                                            node = stack.removeLast();
                                            if (node instanceof $Token) {
                                                $Token token = ($Token) node;
                                                if (firstIndexOfWhiteTerminals <= token.getSymbolID() && token.getSymbolID() < superiorIndexOfWhiteTerminals) {
                                                    list.addFirst(ZERO);
                                                    list.addFirst(node);
                                                    continue;
                                                }
                                            }
                                            break;
                                        }
                                        #if (labelWordCount == 1) {
                                            list.addFirst(new java.lang.Integer(reduction[j]));
                                        #} else {
                                            java.math.BigInteger no = java.math.BigInteger.ZERO;
                                            for (int k = 0; k < labelWordCount; k++)
                                                no = no.or(java.math.BigInteger.valueOf(0xFFFFFFFFL & reduction[j - k]).shiftLeft(k * 32));
                                            list.addFirst(no);
                                        #}
                                        list.addFirst(node);
                                    }
                                    if (isFinal) {
                                        while (!stack.isEmpty()) {
                                            java.lang.Object node = stack.removeLast();
                                            list.addFirst(ZERO);
                                            list.addFirst(node);
                                        }
                                    }
                                    int nodeID = reduction[0];
                                    java.lang.Object node;
                                    if (nodeID <= classSymbolMaximum) {
                                        node = $createNode(nodeID, list);
                                    } else {
                                        node = list;
                                    }
                                    stack.addLast(node);
                                }
                            }
                            return ($Node) stack.getFirst();
                        }
                    #}
                }
                #allocate("binarySearch", topMethodNamespace);
                private static int $binarySearch(int[] table, int key) {
                    // Returns the index where table[i] < key for all i < index
                    int base = 0;
                    int width = table.length >> 1;
                    #java.lang.String lrgLinearSearchThreshold = environment.parameters.getProperty("JavaCodeGenerator.lrgLinearSearchThreshold", "3");
                    while (width > >>[[ lrgLinearSearchThreshold ]]<<) {
                        int halfWidth = width >> 1;
                        int mid = base + width & ~1;    // base + halfWidth * 2
                        if (key <= table[mid]) {
                            width = halfWidth;
                        } else {
                            base = mid;
                            width -= halfWidth;
                        }
                    }
                    return base;
                }
                
                #if (graphStructuredStack) {
                    #reserve("reduce", topMethodNamespace);
                    private static void $reduce(java.util.List$<$GrpahStack> states, $GrpahStack stack, int[] reduction, int index, $ConcreteSyntaxTreeList tail, int[][] lrTable) {
                        if (index == 0) {
                            final int nextSymbolID = reduction[0];
                            $ConcreteSyntaxTree subtree = new $ConcreteSyntaxTree(nextSymbolID, tail);
                            
                            final int returnedStateID = stack.topState();
                            final int[] symbolIDToNextState = lrTable[returnedStateID * 3 + 1];
                            int k = $binarySearch(symbolIDToNextState, nextSymbolID);
                            for (; k < symbolIDToNextState.length; k += 2) {
                                if (symbolIDToNextState[k] > nextSymbolID) {
                                    break;
                                } else if (symbolIDToNextState[k] == nextSymbolID) {
                                    final int nextStateID = symbolIDToNextState[k + 1];
                                    states.add($GrpahStack.push(new $GrpahStack[] { stack }, subtree, nextStateID));
                                    break;
                                }
                            }
                        } else {
                            final int labelWordCount = >>[[ labelWordCount ]]<<;
                            #if (labelWordCount == 1) {
                                java.lang.Integer labels = new java.lang.Integer(reduction[index]);
                            #} else {
                                java.math.BigInteger labels = java.math.BigInteger.ZERO;
                                for (int k = 0; k < labelWordCount; k++)
                                    labels = labels.or(java.math.BigInteger.valueOf(0xFFFFFFFFL & reduction[index - k]).shiftLeft(k * 32));
                            #}
                            $ConcreteSyntaxTreeList list = new $ConcreteSyntaxTreeList(labels, stack.topNode(), tail);
                            $GrpahStack[] prevs = $GrpahStack.pop(stack);
                            for (int k = 0; k < prevs.length; k++) {
                                $reduce(states, prevs[k], reduction, index -= labelWordCount, list, lrTable);
                            }
                        }
                    }
                    private $Node $createNode($ConcreteSyntaxTree subtree, int classSymbolMaximum) throws $ParseException {
                        java.util.List$<$Node> nodeList = new java.util.LinkedList$<$Node>();
                        #if (labelWordCount == 1) {
                            java.util.List$<java.lang.Integer> labelsIDList = new java.util.LinkedList$<java.lang.Integer>();
                            $flatNodes(0, nodeList, labelsIDList, subtree.children, classSymbolMaximum);
                        #} else {
                            java.util.List$<java.math.BigInteger> labelsIDList = new java.util.LinkedList$<java.math.BigInteger>();
                            $flatNodes(java.math.BigInteger.ZERO, nodeList, labelsIDList, subtree.children, classSymbolMaximum);
                        #}
                        $Node[] childNodes = ($Node[]) nodeList.toArray(new $Node[nodeList.size()]);
                        $NodeInitializationParameters parameters = new $NodeInitializationParameters();
                        parameters.childNodes = childNodes;
                        parameters.labelsIDList = labelsIDList;
                        $Node result = $createNode(subtree.symbolID, parameters);
                        return result;
                    }
                    #allocate("flatNodes", topMethodNamespace);
                    #if (labelWordCount == 1) {
                        private void $flatNodes(int labels, java.util.List$<$Node> nodeList, java.util.List$<java.lang.Integer> labelsIDList, $ConcreteSyntaxTreeList list, int classSymbolMaximum) throws $ParseException {
                            for (; list != null; list = list.next) {
                                java.lang.Object node = list.node;
                                java.lang.Integer labelsID = list.labels;
                                int mergedLabelsID = labelsID.intValue();
                                if ((mergedLabelsID & 1) != 0) {
                                    mergedLabelsID &= ~1;
                                    mergedLabelsID |= labels;
                                }
                                if (node instanceof $ConcreteSyntaxTree) {
                                    $ConcreteSyntaxTree subtree = ($ConcreteSyntaxTree) node;
                                    if (subtree.symbolID <= classSymbolMaximum) {
                                        nodeList.add($createNode(subtree, classSymbolMaximum));
                                        labelsIDList.add(new java.lang.Integer(mergedLabelsID));
                                    } else {
                                        $flatNodes(mergedLabelsID, nodeList, labelsIDList, subtree.children, classSymbolMaximum);
                                    }
                                } else {
                                    nodeList.add(($Node) node);
                                    labelsIDList.add(new java.lang.Integer(mergedLabelsID));
                                }
                            }
                        }
                    #} else {
                        private void $flatNodes(java.math.BigInteger labels, java.util.List$<$Node> nodeList, java.util.List$<java.math.BigInteger> labelsIDList, $ConcreteSyntaxTreeList list, int classSymbolMaximum) throws $ParseException {
                            for (; list != null; list = list.next) {
                                java.lang.Object node = list.node;
                                java.math.BigInteger labelsID = list.labels;
                                java.math.BigInteger mergedLabelsID = labelsID;
                                if (mergedLabelsID.testBit(0)) {
                                    mergedLabelsID = mergedLabelsID.clearBit(0);
                                    mergedLabelsID = mergedLabelsID.or(labels);
                                }
                                if (node instanceof $ConcreteSyntaxTree) {
                                    $ConcreteSyntaxTree subtree = ($ConcreteSyntaxTree) node;
                                    if (subtree.symbolID <= classSymbolMaximum) {
                                        nodeList.add($createNode(subtree, classSymbolMaximum));
                                        labelsIDList.add(mergedLabelsID);
                                    } else {
                                        $flatNodes(mergedLabelsID, nodeList, labelsIDList, subtree.children, classSymbolMaximum);
                                    }
                                } else {
                                    nodeList.add(($Node) node);
                                    labelsIDList.add(mergedLabelsID);
                                }
                            }
                        }
                    #}
                #} else {
                    private $Node $createNode(int symbolID, java.util.List data) throws $ParseException {
                        java.util.List$<$Node> nodeList = new java.util.LinkedList$<$Node>();
                        #if (labelWordCount == 1) {
                            java.util.List$<java.lang.Integer> labelsIDList = new java.util.LinkedList$<java.lang.Integer>();
                            $flatNodes(0, nodeList, labelsIDList, data);
                        #} else {
                            java.util.List$<java.math.BigInteger> labelsIDList = new java.util.LinkedList$<java.math.BigInteger>();
                            $flatNodes(java.math.BigInteger.ZERO, nodeList, labelsIDList, data);
                        #}
                        $Node[] childNodes = ($Node[]) nodeList.toArray(new $Node[nodeList.size()]);
                        $NodeInitializationParameters parameters = new $NodeInitializationParameters();
                        parameters.childNodes = childNodes;
                        parameters.labelsIDList = labelsIDList;
                        $Node result = $createNode(symbolID, parameters);
                        return result;
                    }
                    #allocate("flatNodes", topMethodNamespace);
                    #if (labelWordCount == 1) {
                        private static void $flatNodes(int labels, java.util.List$<$Node> nodeList, java.util.List$<java.lang.Integer> labelsIDList, java.util.List data) {
                            java.util.Iterator it = data.iterator();
                            while (it.hasNext()) {
                                java.lang.Object node = it.next();
                                java.lang.Integer labelsID = (java.lang.Integer) it.next();
                                int mergedLabelsID = labelsID.intValue();
                                if ((mergedLabelsID & 1) != 0) {
                                    mergedLabelsID &= ~1;
                                    mergedLabelsID |= labels;
                                }
                                if (node instanceof java.util.List) {
                                    $flatNodes(mergedLabelsID, nodeList, labelsIDList, (java.util.List) node);
                                } else {
                                    nodeList.add(($Node) node);
                                    labelsIDList.add(new java.lang.Integer(mergedLabelsID));
                                }
                            }
                        }
                    #} else {
                        private static void $flatNodes(java.math.BigInteger labels, java.util.List$<$Node> nodeList, java.util.List$<java.math.BigInteger> labelsIDList, java.util.List data) {
                            java.util.Iterator it = data.iterator();
                            while (it.hasNext()) {
                                java.lang.Object node = it.next();
                                java.math.BigInteger labelsID = (java.math.BigInteger) it.next();
                                java.math.BigInteger mergedLabelsID = labelsID;
                                if (mergedLabelsID.testBit(0)) {
                                    mergedLabelsID = mergedLabelsID.clearBit(0);
                                    mergedLabelsID = mergedLabelsID.or(labels);
                                }
                                if (node instanceof java.util.List) {
                                    $flatNodes(mergedLabelsID, nodeList, labelsIDList, (java.util.List) node);
                                } else {
                                    nodeList.add(($Node) node);
                                    labelsIDList.add(mergedLabelsID);
                                }
                            }
                        }
                    #}
                #}
                
                #if (graphStructuredStack) {
                    #allocate("error", topMethodNamespace);
                    private static void $error($LexicalAnalyzer analyzer, int[][] lrTable, int[][] reductions, java.util.List$<$GrpahStack> states, $Token errorNextToken) throws $ParseException {
                        java.util.Set$<java.lang.Integer> nextTokens = new java.util.TreeSet$<java.lang.Integer>();
                        for (int i = 0; i < states.size(); i++) {
                            final $GrpahStack stack = ($GrpahStack) states.get(i);
                            final int stateID = stack.topState();
                            
                            final int[] tokenIDToNextState = lrTable[stateID * 3 + 0];
                            for (int k = 0; k < tokenIDToNextState.length; k += 2)
                                nextTokens.add(new java.lang.Integer(tokenIDToNextState[k]));
                            
                            int[] tokenIDToReductionID = lrTable[stateID * 3 + 2];
                            for (int j = 0; j < tokenIDToReductionID.length; j += 2) {
                                final int reductionID = tokenIDToReductionID[j + 1];
                                final int[] reduction = reductions[reductionID];
                                
                                java.util.List$<$GrpahStack> tmp = new java.util.ArrayList$<$GrpahStack>();
                                $reduce(tmp, stack, reduction, reduction.length - 1, null, lrTable);
                            }
                        }
                        
                        int[] selectableTokens = new int[nextTokens.size()];
                        int index = 0;
                        java.util.Iterator it11 = nextTokens.iterator();
                        while (it11.hasNext()) {
                            java.lang.Integer i = (java.lang.Integer) it11.next();
                            selectableTokens[index++] = i.intValue();
                        }
                        $error(analyzer, errorNextToken, selectableTokens);
                    }
                #} else {
                    #allocate("error", topMethodNamespace);
                    private static void $error($LexicalAnalyzer analyzer, int[][] lrTable, int[][] reductions, java.util.List$<$State> states, $Token errorNextToken) throws $ParseException {
                        java.util.Set$<java.lang.Integer> nextTokens = new java.util.TreeSet$<java.lang.Integer>();
                        for (int i = 0; i < states.size(); i++) {
                            final $State state = ($State) states.get(i);
                            final int stateID = state.stateStack.top();
                            
                            final int[] tokenIDToNextState = lrTable[stateID * 3 + 0];
                            for (int k = 0; k < tokenIDToNextState.length; k += 2)
                                nextTokens.add(new java.lang.Integer(tokenIDToNextState[k]));
                            
                            int[] tokenIDToReductionID = lrTable[stateID * 3 + 2];
                            for (int j = 0; j < tokenIDToReductionID.length; j += 2) {
                                final int reductionID = tokenIDToReductionID[j + 1];
                                final int[] reduction = reductions[reductionID];
                                final int nextSymbolID = reduction[0];
                                
                                $IntegerStack stateStack = state.stateStack;
                                for (int k = reduction.length - 1; k >= 1; k -= >>[[ labelWordCount ]]<<)
                                    stateStack = $IntegerStack.pop(stateStack);
                                
                                $IntegerStack shiftReduceHistory = state.shiftReduceHistory;
                                shiftReduceHistory = $IntegerStack.push(shiftReduceHistory, reductionID);
                                
                                final int returnedStateID = stateStack.top();
                                final int[] symbolIDToNextState = lrTable[returnedStateID * 3 + 1];
                                for (int k = 0; k < symbolIDToNextState.length; k += 2) {
                                    if (symbolIDToNextState[k] == nextSymbolID) {
                                        final int nextStateID = symbolIDToNextState[k + 1];
                                        states.add(new $State($IntegerStack.push(stateStack, nextStateID), shiftReduceHistory));
                                        break;
                                    }
                                }
                            }
                        }
                        
                        int[] selectableTokens = new int[nextTokens.size()];
                        int index = 0;
                        java.util.Iterator it11 = nextTokens.iterator();
                        while (it11.hasNext()) {
                            java.lang.Integer i = (java.lang.Integer) it11.next();
                            selectableTokens[index++] = i.intValue();
                        }
                        $error(analyzer, errorNextToken, selectableTokens);
                    }
                #}
                /**
                 * Throws or recovers a error.
                 * <p>
                 * This method is called when a token is not shiftable.
                 * If the error is not recoverable, this method throws $ParseException.
                 * To recover the error, this method dose not throw anything,
                 * then syntax analyzers ignore the <code>errorNextToken</code>,
                 * i.e. the last <code>analyzer.{@link $Parser.$LexicalAnalyzer#next() next()}</code> call.
                 * The next <code>analyzer.{@link $Parser.$LexicalAnalyzer#next() next()}</code> can return a better token.
                 * 
                 * @param   analyzer
                 *      the lexical analyzer of the parsing.
                 * @param   errorNextToken
                 *      the unshiftable token the last <code>analyzer.{@link $Parser.$LexicalAnalyzer#next() next()}</code> returned.
                 * @param   selectableNextTokens
                 *      the shiftable tokens at the situation.
                 * @throws   $ParseException
                 *      if the error is not recoverable.
                 * @since    notavaCC 1.0
                 */
                private static void $error($LexicalAnalyzer analyzer, $Token errorNextToken, int[] selectableNextTokens) throws $ParseException {
                    java.io.StringWriter buffer = new java.io.StringWriter();
                    java.io.PrintWriter writer = new java.io.PrintWriter(buffer);
                    
                    if (errorNextToken.getSymbolID() != EOF_TOKEN) {
                        writer.println("The token " + errorNextToken + " should be one of the followings.");
                    } else {
                        writer.println("The EOF should be one of the followings.");
                    }
                    
                    final java.lang.String[] tokenStrings = {
                        #{
                            #Symbol[] symbols = tables.symbols();
                            #for (int i = 0; i < tables.superiorIndexOfTerminals(); i++) {
                                #Symbol symbol = symbols[i];
                                >>[[ quoted(symbol.toString()) ]]<<,
                            #}
                        #}
                    };
                    
                    for (int i = 0; i < selectableNextTokens.length; i++) {
                        int selectableNextToken = selectableNextTokens[i];
                        writer.print("        ");
                        writer.println(tokenStrings[selectableNextToken]);
                    }
                    
                    writer.close();
                    if (writer.checkError())
                        throw new RuntimeException();
                    throw new $ParseException(buffer.toString(), errorNextToken);
                }
            #}
            
            #{
                #// DFA table
                #final int unit = 50;
                #DFAState[] states = tables.dfaStates();
                #logger.verbose(environment.processingFile, "DFA table: ''{0}'' entries.", new Integer(states.length));
                private static $final_avoid_jdk12_bug int[][] $dfaTable;
                #allocate("dfaTable", topFieldNamespace);
                static {
                    $dfaTable = new int[>>[[ states.length ]]<<][];
                    #for (int i = 0; i < states.length; i += unit) {
                        initializeDFATable>>[[ i ]]<<();
                    #}
                }
                #for (int i = 0; i < states.length; i += unit) {
                    #topMethodNamespace.add("initializeDFATable" + i, RESERVED_NAME);
                    private static void initializeDFATable>>[[ i ]]<<() {
                        #for (int j = i; j < i + unit && j < states.length; j++) {
                            #DFAState state = states[j];
                            $dfaTable[>>[[ j ]]<<] = new int[] {
                                #Terminal symbol = state.hitSymbol();
                                #if (symbol == null)
                                    -1,
                                #else
                                    >>[[ symbolToJavaWord.get(symbol) ]]<<,
                                #Iterator it = state.getMapInputToNextStates().entrySet().iterator();
                                #while (it.hasNext()) {
                                    #Map.Entry entry = (Map.Entry) it.next();
                                    #DFAInput input = (DFAInput) entry.getKey();
                                    #DFAState nextState = (DFAState) entry.getValue();
                                    >>[[ quoted(input.lower) ]]<<, >>[[ quoted(input.upper) ]]<<, >>[[ tables.getIndexOfState(nextState) ]]<<, 
                                #}
                            };
                        #}
                    }
                #}
            #}
            
            #{
                #// LALR(1) table
                #final int unit = 20;
                #LRState[] states = tables.lrStates();
                #logger.verbose(environment.processingFile, "LR table: ''{0}'' entries.", new Integer(states.length));
                #int tableSize = 0;
                private static $final_avoid_jdk12_bug int[][] $lrTable;
                #allocate("lrTable", topFieldNamespace);
                static {
                    $lrTable = new int[>>[[ states.length ]]<< * 3][];
                    #for (int i = 0; i < states.length; i += unit) {
                        initializeLRTable>>[[ i ]]<<();
                    #}
                }
                #for (int i = 0; i < states.length; i += unit) {
                    #topMethodNamespace.add("initializeLRTable" + i, RESERVED_NAME);
                    private static void initializeLRTable>>[[ i ]]<<() {
                        #for (int j = i; j < i + unit && j < states.length; j++) {
                            #LRState state = states[j];
                            #Map teminalToNextState = tables.getMapTeminalToNextState(state);
                            #Map nonteminalToNextState = tables.getMapNonteminalToNextState(state);
                            #Map symbolToReductions = tables.getMapSymbolToReduction(state);
                            // LRState >>[[ tables.getIndexOfState(state) ]]<<
                            #tableSize += 4;    // address
                            $lrTable[>>[[j]]<< * 3 + 0] = new int[] {
                                #Iterator it3 = teminalToNextState.entrySet().iterator();
                                #while (it3.hasNext()) {
                                    #Map.Entry entry = (Map.Entry) it3.next();
                                    #Symbol symbol = (Symbol) entry.getKey();
                                    #LRState nextState = (LRState) entry.getValue();
                                    >>[[ symbolToJavaWord.get(symbol) ]]<<, >>[[ tables.getIndexOfState(nextState) ]]<<,
                                    #tableSize += 4 + 4;
                                #}
                            };
                            #tableSize += 4;    // address
                            $lrTable[>>[[j]]<< * 3 + 1] = new int[] {
                                #Iterator it4 = nonteminalToNextState.entrySet().iterator();
                                #while (it4.hasNext()) {
                                    #Map.Entry entry = (Map.Entry) it4.next();
                                    #Symbol symbol = (Symbol) entry.getKey();
                                    #LRState nextState = (LRState) entry.getValue();
                                    >>[[ symbolToJavaWord.get(symbol) ]]<<, >>[[ tables.getIndexOfState(nextState) ]]<<,
                                    #tableSize += 4 + 4;
                                #}
                            };
                            #tableSize += 4;    // address
                            $lrTable[>>[[j]]<< * 3 + 2] = new int[] {
                                #Iterator it5 = symbolToReductions.entrySet().iterator();
                                #while (it5.hasNext()) {
                                    #Map.Entry entry = (Map.Entry) it5.next();
                                    #Symbol symbol = (Symbol) entry.getKey();
                                    #Set reductions = (Set) entry.getValue();
                                    #Iterator it6 = reductions.iterator();
                                    #while (it6.hasNext()) {
                                        #Reduction reduction = (Reduction) it6.next();
                                        >>[[ symbolToJavaWord.get(symbol) ]]<<, >>[[ tables.getIndexOfReduction(reduction) ]]<<,
                                        #tableSize += 4 + 4;
                                    #}
                                #}
                            };
                        #}
                    }
                #}
                #logger.verbose(environment.processingFile, "LR table: ''{0}'' bytes.", new Integer(tableSize));
            #}
            
            #{
                #// reduction table
                #Reduction[] reductions = tables.reductions();
                #logger.verbose(environment.processingFile, "''{0}'' reductions.", new Integer(reductions.length));
                #allocate("reductionTable", topFieldNamespace);
                private static final int[][] $reductionTable = {
                    #for (int i = 0; i < reductions.length; i++) {
                        #Reduction reduction = reductions[i];
                        
                        #java.lang.StringBuffer result = new java.lang.StringBuffer();
                        #Iterator it = reduction.labelSetList().iterator();
                        #while (it.hasNext()) {
                            #Set labels = (Set) it.next();
                            #BigInteger no = BigInteger.ZERO;
                            #Iterator innest = labels.iterator();
                            #while (innest.hasNext()) {
                                #java.lang.String label = (String) innest.next();
                                #no = no.setBit(tables.getIndexOfLabel(label));
                            #}
                            #String hex = no.toString(16).toUpperCase(Locale.ENGLISH);
                            #while (hex.length() < labelWordCount * 8) {
                                #hex = '0' + hex;
                            #}
                            #for (int j = 0; j < labelWordCount; j++) {
                                #result.append("0x").append(hex.substring(j * 8, j * 8 + 8)).append(",");
                            #}
                            #result.append(" ");
                        #}
                        { >>[[ symbolToJavaWord.get(reduction.reducedSymbol()) ]]<<,  >>[[ result ]]<<},
                    #}
                };
            #}
            
            #{
                #// tag table
                #allocate("tagTable", topFieldNamespace);
                private static final int[] $tagTable = {
                    #LRState[] states = tables.lrStates();
                    #for (int i = 0; i < states.length; i++) {
                        #LRState state = states[i];
                        #if (state.tag() != null)
                            >>[[ i ]]<<, >>[[ tagToWord.get(state.tag().getImage()) ]]<<,
                    #}
                };
            #}
            
            #allocate("replaceAll", topMethodNamespace);
            private static $<T> void $replaceAll(java.util.List$<T> list, $Node oldValue, >>[[ targetVersion < 1.5 ? "$Node" : "T" ]]<< newValue) {
                #if (1.4 <= targetVersion && targetVersion < 1.5) {
                    java.util.Collections.replaceAll(list, oldValue, newValue);
                #} else {
                    java.util.ListIterator$<T> it = list.listIterator();
                    if (oldValue == null) {
                        while (it.hasNext()) {
                            java.lang.Object old = it.next();
                            if (oldValue == old)
                                it.set(newValue);
                        }
                    } else {
                        while (it.hasNext()) {
                            java.lang.Object old = it.next();
                            if (oldValue.equals(old))
                                it.set(newValue);
                        }
                    }
                #}
            }
            
            #allocate("toHexCode", topMethodNamespace);
            private static java.lang.String $toHexCode(char ch) {
                java.lang.String hex = "000" + java.lang.Integer.toString(ch, 16).toUpperCase(java.util.Locale.ENGLISH);
                hex = hex.substring(hex.length() - 4);
                return hex;
            }
#//            #allocate("quoted", topMethodNamespace);
#//            private static java.lang.String $quoted(java.lang.String str, boolean quote) {
#//                java.lang.StringBuffer quoted = new java.lang.StringBuffer();
#//                if (quote)
#//                    quoted.append("\"");
#//                for (int i = 0; i < str.length(); i++) {
#//                    char ch = str.charAt(i);
#//                    $quoted(quoted, ch);
#//                }
#//                if (quote)
#//                    quoted.append("\"");
#//                return quoted.toString();
#//            }
#//            #allocate("quoted", topMethodNamespace);
#//            private static void $quoted(java.lang.StringBuffer result, char ch) {
#//                switch (ch) {
#//                default:
#//                    if (0x0020 <= ch && ch <= 0x007E) {
#//                        result.append(ch);
#//                    } else {
#//                        result.append("\\u").append($toHexCode(ch));
#//                    }
#//                    break;
#//                case '\n':   result.append("\\n");      break;
#//                case '\t':   result.append("\\t");      break;
#//                case '\b':   result.append("\\b");      break;
#//                case '\r':   result.append("\\r");      break;
#//                case '\f':   result.append("\\f");      break;
#//                case '\\':   result.append("\\\\");     break;
#//                case '\'':   result.append("\\\'");     break;
#//                case '\"':   result.append("\\\"");     break;
#//                }
#//            }
            
            #if (!root.hasProtectedConstructor() && publicParsableTypeCount == 1) {
                #if (manualLocale.equals("ja")) {
                    /**
                    #if (javadocForManual) {
                         * 引数に渡されたファイルをパーズし、構文木を出力します（サンプルプログラム）。
                         * <p>
                         * このメソッドは、<code>$parsable</code> な非終端記号が１つ存在する場合にだけ出力されます。このメソッドの仕様は、将来のバージョンでは変更になる可能性があります。 
                    #} else {
                         * 引数に渡されたファイルをパーズし結果を出力します。
                    #}
                     * 
                     * @param   args
                     *      解析対象のファイルの名前
                    #if (javadocForManual) {
                        * @since    notavaCC 1.0
                    #}
                     */
                #} else {
                    /**
                    #if (javadocForManual) {
                         * Parses the given files and prints the syntax tree (a sample program).
                         * <p>
                         * This method is generated if there is only one <code>$parsable</code> nonterminal symbol.  The specification of this method may be changed in the feature.
                    #} else {
                         * Parses the given files and prints the syntax tree.
                    #}
                     * 
                     * @param   args
                     *      the names of the files to parse.
                    #if (javadocForManual) {
                        * @since    notavaCC 1.0
                    #}
                     */
                #}
                #reserve("main", topMethodNamespace);
                public static void $main(java.lang.String[] args) {
                    try {
                        $Parser p = new $Parser();
                        for (int i = 0; i < args.length; i++) {
                            java.io.File file = new java.io.File(args[i]);
                            System.out.println(file);
                            
                            #Iterator it12 = tables.userDefinedTypes().iterator();
                            #while (it12.hasNext()) {
                                #UserDefinedType type = (UserDefinedType) it12.next();
                                #LRState initialState = (LRState) lrTable.symbolToInitialState().get(type.getTypeNonterminal());
                                #if (initialState != null) {
                                    System.out.println(p.parse>>[[ type.getName() ]]<<(file));
                                    #break;
                                #}
                            #}
                        }
                    } catch(java.lang.Exception x) {
                        x.printStackTrace();
                    }
                }
            #}
        } // class $Parser
>>[[
        
        logger.verbosest(environment.processingFile, "checking topTypeNamespace");
        checkDuplex(topTypeNamespace);
        logger.verbosest(environment.processingFile, "checking topFieldNamespace");
        checkDuplex(topFieldNamespace);
        logger.verbosest(environment.processingFile, "checking topMethodNamespace");
        checkDuplex(topMethodNamespace);
        logger.verbosest(environment.processingFile, "checking nodeFieldNamespace");
        checkDuplex(nodeFieldNamespace);
        logger.verbosest(environment.processingFile, "checking nodeNoArgumentMethodNamespace");
        checkDuplex(nodeNoArgumentMethodNamespace);
        logger.verbosest(environment.processingFile, "checking nodeMethodNamespace");
        checkDuplex(nodeMethodNamespace);
        if (logger.hasError())
            logger.fatal();
    } // generateCode
    public void checkDuplex(MultiMap namespace) {
        Iterator it13 = namespace.entrySet().iterator();
        while (it13.hasNext()) {
            Map.Entry entry = (Map.Entry) it13.next();
            String name = (String) entry.getKey();
            Set reservers = (Set) entry.getValue();
            if (reservers.contains(RESERVED_NAME))
                logger.verbosest(environment.processingFile, "checking the reserved name {0}.", name);
            if (reservers.size() >= 2) {
                Iterator it14 = reservers.iterator();
                while (it14.hasNext()) {
                    Object reserver = (Object) it14.next();
                    if (reserver.equals(RESERVED_NAME)) {
                        // ignore.
                    } else if (reserver instanceof Symbol) {
                        logger.error(environment.processingFile, "The reserved name {0} is used.", reserver);
                    } else if (reserver instanceof Type) {
                        logger.error(environment.processingFile, "The reserved name {0} is used.", reserver);
                    } else {
                        logger.error(environment.processingFile, "The reserved name {0} is used.", reserver);
                    }
                }
            }
        }
    }
} // JavaCodeGenerator

